# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
# This file is machine-generated by lib/unicore/mktables from the Unicode
# database, Version 16.0.0.  Any changes made here will be lost!

# !!!!!!!   INTERNAL PERL USE ONLY   !!!!!!!
# This file is for internal use by core Perl only.  The format and even the
# name or existence of this file are subject to change without notice.  Don't
# use it directly.  Use Unicode::UCD to access the Unicode character data
# base.



# The name this table is to be known by, with the format of the mappings in
# the main body of the table, and what all code points missing from this file
# map to.
$Unicode::UCD::SwashInfo{'ToWB'}{'format'} = 's'; # string
$Unicode::UCD::SwashInfo{'ToWB'}{'missing'} = 'Other';

return <<'END';
9		Perl_Tailored_HSpace
A		LF
B	C	Newline
D		CR
20		Perl_Tailored_HSpace
22		Double_Quote
27		Single_Quote
2C		MidNum
2E		MidNumLet
30	39	Numeric
3A		MidLetter
3B		MidNum
41	5A	ALetter
5F		ExtendNumLet
61	7A	ALetter
85		Newline
A0		Perl_Tailored_HSpace
A9		ExtPict_XX
AA		ALetter
AD		Format
AE		ExtPict_XX
B5		ALetter
B7		MidLetter
BA		ALetter
C0	D6	ALetter
D8	F6	ALetter
F8	2D7	ALetter
2DE	2FF	ALetter
300	36F	Extend
370	374	ALetter
376	377	ALetter
37A	37D	ALetter
37E		MidNum
37F		ALetter
386		ALetter
387		MidLetter
388	38A	ALetter
38C		ALetter
38E	3A1	ALetter
3A3	3F5	ALetter
3F7	481	ALetter
483	489	Extend
48A	52F	ALetter
531	556	ALetter
559	55C	ALetter
55E		ALetter
55F		MidLetter
560	588	ALetter
589		MidNum
58A		ALetter
591	5BD	Extend
5BF		Extend
5C1	5C2	Extend
5C4	5C5	Extend
5C7		Extend
5D0	5EA	Hebrew_Letter
5EF	5F2	Hebrew_Letter
5F3		ALetter
5F4		MidLetter
600	605	Numeric
60C	60D	MidNum
610	61A	Extend
61C		Format
620	64A	ALetter
64B	65F	Extend
660	669	Numeric
66B		Numeric
66C		MidNum
66E	66F	ALetter
670		Extend
671	6D3	ALetter
6D5		ALetter
6D6	6DC	Extend
6DD		Numeric
6DF	6E4	Extend
6E5	6E6	ALetter
6E7	6E8	Extend
6EA	6ED	Extend
6EE	6EF	ALetter
6F0	6F9	Numeric
6FA	6FC	ALetter
6FF		ALetter
70F	710	ALetter
711		Extend
712	72F	ALetter
730	74A	Extend
74D	7A5	ALetter
7A6	7B0	Extend
7B1		ALetter
7C0	7C9	Numeric
7CA	7EA	ALetter
7EB	7F3	Extend
7F4	7F5	ALetter
7F8		MidNum
7FA		ALetter
7FD		Extend
800	815	ALetter
816	819	Extend
81A		ALetter
81B	823	Extend
824		ALetter
825	827	Extend
828		ALetter
829	82D	Extend
840	858	ALetter
859	85B	Extend
860	86A	ALetter
870	887	ALetter
889	88E	ALetter
890	891	Numeric
897	89F	Extend
8A0	8C9	ALetter
8CA	8E1	Extend
8E2		Numeric
8E3	903	Extend
904	939	ALetter
93A	93C	Extend
93D		ALetter
93E	94F	Extend
950		ALetter
951	957	Extend
958	961	ALetter
962	963	Extend
966	96F	Numeric
971	980	ALetter
981	983	Extend
985	98C	ALetter
98F	990	ALetter
993	9A8	ALetter
9AA	9B0	ALetter
9B2		ALetter
9B6	9B9	ALetter
9BC		Extend
9BD		ALetter
9BE	9C4	Extend
9C7	9C8	Extend
9CB	9CD	Extend
9CE		ALetter
9D7		Extend
9DC	9DD	ALetter
9DF	9E1	ALetter
9E2	9E3	Extend
9E6	9EF	Numeric
9F0	9F1	ALetter
9FC		ALetter
9FE		Extend
A01	A03	Extend
A05	A0A	ALetter
A0F	A10	ALetter
A13	A28	ALetter
A2A	A30	ALetter
A32	A33	ALetter
A35	A36	ALetter
A38	A39	ALetter
A3C		Extend
A3E	A42	Extend
A47	A48	Extend
A4B	A4D	Extend
A51		Extend
A59	A5C	ALetter
A5E		ALetter
A66	A6F	Numeric
A70	A71	Extend
A72	A74	ALetter
A75		Extend
A81	A83	Extend
A85	A8D	ALetter
A8F	A91	ALetter
A93	AA8	ALetter
AAA	AB0	ALetter
AB2	AB3	ALetter
AB5	AB9	ALetter
ABC		Extend
ABD		ALetter
ABE	AC5	Extend
AC7	AC9	Extend
ACB	ACD	Extend
AD0		ALetter
AE0	AE1	ALetter
AE2	AE3	Extend
AE6	AEF	Numeric
AF9		ALetter
AFA	AFF	Extend
B01	B03	Extend
B05	B0C	ALetter
B0F	B10	ALetter
B13	B28	ALetter
B2A	B30	ALetter
B32	B33	ALetter
B35	B39	ALetter
B3C		Extend
B3D		ALetter
B3E	B44	Extend
B47	B48	Extend
B4B	B4D	Extend
B55	B57	Extend
B5C	B5D	ALetter
B5F	B61	ALetter
B62	B63	Extend
B66	B6F	Numeric
B71		ALetter
B82		Extend
B83		ALetter
B85	B8A	ALetter
B8E	B90	ALetter
B92	B95	ALetter
B99	B9A	ALetter
B9C		ALetter
B9E	B9F	ALetter
BA3	BA4	ALetter
BA8	BAA	ALetter
BAE	BB9	ALetter
BBE	BC2	Extend
BC6	BC8	Extend
BCA	BCD	Extend
BD0		ALetter
BD7		Extend
BE6	BEF	Numeric
C00	C04	Extend
C05	C0C	ALetter
C0E	C10	ALetter
C12	C28	ALetter
C2A	C39	ALetter
C3C		Extend
C3D		ALetter
C3E	C44	Extend
C46	C48	Extend
C4A	C4D	Extend
C55	C56	Extend
C58	C5A	ALetter
C5D		ALetter
C60	C61	ALetter
C62	C63	Extend
C66	C6F	Numeric
C80		ALetter
C81	C83	Extend
C85	C8C	ALetter
C8E	C90	ALetter
C92	CA8	ALetter
CAA	CB3	ALetter
CB5	CB9	ALetter
CBC		Extend
CBD		ALetter
CBE	CC4	Extend
CC6	CC8	Extend
CCA	CCD	Extend
CD5	CD6	Extend
CDD	CDE	ALetter
CE0	CE1	ALetter
CE2	CE3	Extend
CE6	CEF	Numeric
CF1	CF2	ALetter
CF3		Extend
D00	D03	Extend
D04	D0C	ALetter
D0E	D10	ALetter
D12	D3A	ALetter
D3B	D3C	Extend
D3D		ALetter
D3E	D44	Extend
D46	D48	Extend
D4A	D4D	Extend
D4E		ALetter
D54	D56	ALetter
D57		Extend
D5F	D61	ALetter
D62	D63	Extend
D66	D6F	Numeric
D7A	D7F	ALetter
D81	D83	Extend
D85	D96	ALetter
D9A	DB1	ALetter
DB3	DBB	ALetter
DBD		ALetter
DC0	DC6	ALetter
DCA		Extend
DCF	DD4	Extend
DD6		Extend
DD8	DDF	Extend
DE6	DEF	Numeric
DF2	DF3	Extend
E31		Extend
E34	E3A	Extend
E47	E4E	Extend
E50	E59	Numeric
EB1		Extend
EB4	EBC	Extend
EC8	ECE	Extend
ED0	ED9	Numeric
F00		ALetter
F18	F19	Extend
F20	F29	Numeric
F35		Extend
F37		Extend
F39		Extend
F3E	F3F	Extend
F40	F47	ALetter
F49	F6C	ALetter
F71	F84	Extend
F86	F87	Extend
F88	F8C	ALetter
F8D	F97	Extend
F99	FBC	Extend
FC6		Extend
102B	103E	Extend
1040	1049	Numeric
1056	1059	Extend
105E	1060	Extend
1062	1064	Extend
1067	106D	Extend
1071	1074	Extend
1082	108D	Extend
108F		Extend
1090	1099	Numeric
109A	109D	Extend
10A0	10C5	ALetter
10C7		ALetter
10CD		ALetter
10D0	10FA	ALetter
10FC	1248	ALetter
124A	124D	ALetter
1250	1256	ALetter
1258		ALetter
125A	125D	ALetter
1260	1288	ALetter
128A	128D	ALetter
1290	12B0	ALetter
12B2	12B5	ALetter
12B8	12BE	ALetter
12C0		ALetter
12C2	12C5	ALetter
12C8	12D6	ALetter
12D8	1310	ALetter
1312	1315	ALetter
1318	135A	ALetter
135D	135F	Extend
1380	138F	ALetter
13A0	13F5	ALetter
13F8	13FD	ALetter
1401	166C	ALetter
166F	167F	ALetter
1680		Perl_Tailored_HSpace
1681	169A	ALetter
16A0	16EA	ALetter
16EE	16F8	ALetter
1700	1711	ALetter
1712	1715	Extend
171F	1731	ALetter
1732	1734	Extend
1740	1751	ALetter
1752	1753	Extend
1760	176C	ALetter
176E	1770	ALetter
1772	1773	Extend
17B4	17D3	Extend
17DD		Extend
17E0	17E9	Numeric
180B	180D	Extend
180E		Format
180F		Extend
1810	1819	Numeric
1820	1878	ALetter
1880	1884	ALetter
1885	1886	Extend
1887	18A8	ALetter
18A9		Extend
18AA		ALetter
18B0	18F5	ALetter
1900	191E	ALetter
1920	192B	Extend
1930	193B	Extend
1946	194F	Numeric
19D0	19DA	Numeric
1A00	1A16	ALetter
1A17	1A1B	Extend
1A55	1A5E	Extend
1A60	1A7C	Extend
1A7F		Extend
1A80	1A89	Numeric
1A90	1A99	Numeric
1AB0	1ACE	Extend
1B00	1B04	Extend
1B05	1B33	ALetter
1B34	1B44	Extend
1B45	1B4C	ALetter
1B50	1B59	Numeric
1B6B	1B73	Extend
1B80	1B82	Extend
1B83	1BA0	ALetter
1BA1	1BAD	Extend
1BAE	1BAF	ALetter
1BB0	1BB9	Numeric
1BBA	1BE5	ALetter
1BE6	1BF3	Extend
1C00	1C23	ALetter
1C24	1C37	Extend
1C40	1C49	Numeric
1C4D	1C4F	ALetter
1C50	1C59	Numeric
1C5A	1C7D	ALetter
1C80	1C8A	ALetter
1C90	1CBA	ALetter
1CBD	1CBF	ALetter
1CD0	1CD2	Extend
1CD4	1CE8	Extend
1CE9	1CEC	ALetter
1CED		Extend
1CEE	1CF3	ALetter
1CF4		Extend
1CF5	1CF6	ALetter
1CF7	1CF9	Extend
1CFA		ALetter
1D00	1DBF	ALetter
1DC0	1DFF	Extend
1E00	1F15	ALetter
1F18	1F1D	ALetter
1F20	1F45	ALetter
1F48	1F4D	ALetter
1F50	1F57	ALetter
1F59		ALetter
1F5B		ALetter
1F5D		ALetter
1F5F	1F7D	ALetter
1F80	1FB4	ALetter
1FB6	1FBC	ALetter
1FBE		ALetter
1FC2	1FC4	ALetter
1FC6	1FCC	ALetter
1FD0	1FD3	ALetter
1FD6	1FDB	ALetter
1FE0	1FEC	ALetter
1FF2	1FF4	ALetter
1FF6	1FFC	ALetter
2000	200A	Perl_Tailored_HSpace
200C		Extend
200D		ZWJ
200E	200F	Format
2018	2019	MidNumLet
2024		MidNumLet
2027		MidLetter
2028	2029	Newline
202A	202E	Format
202F		ExtendNumLet
203C		ExtPict_XX
203F	2040	ExtendNumLet
2044		MidNum
2049		ExtPict_XX
2054		ExtendNumLet
205F		Perl_Tailored_HSpace
2060	2064	Format
2066	206F	Format
2071		ALetter
207F		ALetter
2090	209C	ALetter
20D0	20F0	Extend
2102		ALetter
2107		ALetter
210A	2113	ALetter
2115		ALetter
2119	211D	ALetter
2122		ExtPict_XX
2124		ALetter
2126		ALetter
2128		ALetter
212A	212D	ALetter
212F	2138	ALetter
2139		ExtPict_LE
213C	213F	ALetter
2145	2149	ALetter
214E		ALetter
2160	2188	ALetter
2194	2199	ExtPict_XX
21A9	21AA	ExtPict_XX
231A	231B	ExtPict_XX
2328		ExtPict_XX
2388		ExtPict_XX
23CF		ExtPict_XX
23E9	23F3	ExtPict_XX
23F8	23FA	ExtPict_XX
24B6	24C1	ALetter
24C2		ExtPict_LE
24C3	24E9	ALetter
25AA	25AB	ExtPict_XX
25B6		ExtPict_XX
25C0		ExtPict_XX
25FB	25FE	ExtPict_XX
2600	2605	ExtPict_XX
2607	2612	ExtPict_XX
2614	2685	ExtPict_XX
2690	2705	ExtPict_XX
2708	2712	ExtPict_XX
2714		ExtPict_XX
2716		ExtPict_XX
271D		ExtPict_XX
2721		ExtPict_XX
2728		ExtPict_XX
2733	2734	ExtPict_XX
2744		ExtPict_XX
2747		ExtPict_XX
274C		ExtPict_XX
274E		ExtPict_XX
2753	2755	ExtPict_XX
2757		ExtPict_XX
2763	2767	ExtPict_XX
2795	2797	ExtPict_XX
27A1		ExtPict_XX
27B0		ExtPict_XX
27BF		ExtPict_XX
2934	2935	ExtPict_XX
2B05	2B07	ExtPict_XX
2B1B	2B1C	ExtPict_XX
2B50		ExtPict_XX
2B55		ExtPict_XX
2C00	2CE4	ALetter
2CEB	2CEE	ALetter
2CEF	2CF1	Extend
2CF2	2CF3	ALetter
2D00	2D25	ALetter
2D27		ALetter
2D2D		ALetter
2D30	2D67	ALetter
2D6F		ALetter
2D7F		Extend
2D80	2D96	ALetter
2DA0	2DA6	ALetter
2DA8	2DAE	ALetter
2DB0	2DB6	ALetter
2DB8	2DBE	ALetter
2DC0	2DC6	ALetter
2DC8	2DCE	ALetter
2DD0	2DD6	ALetter
2DD8	2DDE	ALetter
2DE0	2DFF	Extend
2E2F		ALetter
3000		Perl_Tailored_HSpace
3005		ALetter
302A	302F	Extend
3030		ExtPict_XX
3031	3035	Katakana
303B	303C	ALetter
303D		ExtPict_XX
3099	309A	Extend
309B	309C	Katakana
30A0	30FA	Katakana
30FC	30FF	Katakana
3105	312F	ALetter
3131	318E	ALetter
31A0	31BF	ALetter
31F0	31FF	Katakana
3297		ExtPict_XX
3299		ExtPict_XX
32D0	32FE	Katakana
3300	3357	Katakana
A000	A48C	ALetter
A4D0	A4FD	ALetter
A500	A60C	ALetter
A610	A61F	ALetter
A620	A629	Numeric
A62A	A62B	ALetter
A640	A66E	ALetter
A66F	A672	Extend
A674	A67D	Extend
A67F	A69D	ALetter
A69E	A69F	Extend
A6A0	A6EF	ALetter
A6F0	A6F1	Extend
A708	A7CD	ALetter
A7D0	A7D1	ALetter
A7D3		ALetter
A7D5	A7DC	ALetter
A7F2	A801	ALetter
A802		Extend
A803	A805	ALetter
A806		Extend
A807	A80A	ALetter
A80B		Extend
A80C	A822	ALetter
A823	A827	Extend
A82C		Extend
A840	A873	ALetter
A880	A881	Extend
A882	A8B3	ALetter
A8B4	A8C5	Extend
A8D0	A8D9	Numeric
A8E0	A8F1	Extend
A8F2	A8F7	ALetter
A8FB		ALetter
A8FD	A8FE	ALetter
A8FF		Extend
A900	A909	Numeric
A90A	A925	ALetter
A926	A92D	Extend
A930	A946	ALetter
A947	A953	Extend
A960	A97C	ALetter
A980	A983	Extend
A984	A9B2	ALetter
A9B3	A9C0	Extend
A9CF		ALetter
A9D0	A9D9	Numeric
A9E5		Extend
A9F0	A9F9	Numeric
AA00	AA28	ALetter
AA29	AA36	Extend
AA40	AA42	ALetter
AA43		Extend
AA44	AA4B	ALetter
AA4C	AA4D	Extend
AA50	AA59	Numeric
AA7B	AA7D	Extend
AAB0		Extend
AAB2	AAB4	Extend
AAB7	AAB8	Extend
AABE	AABF	Extend
AAC1		Extend
AAE0	AAEA	ALetter
AAEB	AAEF	Extend
AAF2	AAF4	ALetter
AAF5	AAF6	Extend
AB01	AB06	ALetter
AB09	AB0E	ALetter
AB11	AB16	ALetter
AB20	AB26	ALetter
AB28	AB2E	ALetter
AB30	AB69	ALetter
AB70	ABE2	ALetter
ABE3	ABEA	Extend
ABEC	ABED	Extend
ABF0	ABF9	Numeric
AC00	D7A3	ALetter
D7B0	D7C6	ALetter
D7CB	D7FB	ALetter
FB00	FB06	ALetter
FB13	FB17	ALetter
FB1D		Hebrew_Letter
FB1E		Extend
FB1F	FB28	Hebrew_Letter
FB2A	FB36	Hebrew_Letter
FB38	FB3C	Hebrew_Letter
FB3E		Hebrew_Letter
FB40	FB41	Hebrew_Letter
FB43	FB44	Hebrew_Letter
FB46	FB4F	Hebrew_Letter
FB50	FBB1	ALetter
FBD3	FD3D	ALetter
FD50	FD8F	ALetter
FD92	FDC7	ALetter
FDF0	FDFB	ALetter
FE00	FE0F	Extend
FE13		MidLetter
FE20	FE2F	Extend
FE33	FE34	ExtendNumLet
FE4D	FE4F	ExtendNumLet
FE50		MidNum
FE52		MidNumLet
FE54		MidNum
FE55		MidLetter
FE70	FE74	ALetter
FE76	FEFC	ALetter
FEFF		Format
FF07		MidNumLet
FF0C		MidNum
FF0E		MidNumLet
FF10	FF19	Numeric
FF1A		MidLetter
FF1B		MidNum
FF21	FF3A	ALetter
FF3F		ExtendNumLet
FF41	FF5A	ALetter
FF66	FF9D	Katakana
FF9E	FF9F	Extend
FFA0	FFBE	ALetter
FFC2	FFC7	ALetter
FFCA	FFCF	ALetter
FFD2	FFD7	ALetter
FFDA	FFDC	ALetter
FFF9	FFFB	Format
10000	1000B	ALetter
1000D	10026	ALetter
10028	1003A	ALetter
1003C	1003D	ALetter
1003F	1004D	ALetter
10050	1005D	ALetter
10080	100FA	ALetter
10140	10174	ALetter
101FD		Extend
10280	1029C	ALetter
102A0	102D0	ALetter
102E0		Extend
10300	1031F	ALetter
1032D	1034A	ALetter
10350	10375	ALetter
10376	1037A	Extend
10380	1039D	ALetter
103A0	103C3	ALetter
103C8	103CF	ALetter
103D1	103D5	ALetter
10400	1049D	ALetter
104A0	104A9	Numeric
104B0	104D3	ALetter
104D8	104FB	ALetter
10500	10527	ALetter
10530	10563	ALetter
10570	1057A	ALetter
1057C	1058A	ALetter
1058C	10592	ALetter
10594	10595	ALetter
10597	105A1	ALetter
105A3	105B1	ALetter
105B3	105B9	ALetter
105BB	105BC	ALetter
105C0	105F3	ALetter
10600	10736	ALetter
10740	10755	ALetter
10760	10767	ALetter
10780	10785	ALetter
10787	107B0	ALetter
107B2	107BA	ALetter
10800	10805	ALetter
10808		ALetter
1080A	10835	ALetter
10837	10838	ALetter
1083C		ALetter
1083F	10855	ALetter
10860	10876	ALetter
10880	1089E	ALetter
108E0	108F2	ALetter
108F4	108F5	ALetter
10900	10915	ALetter
10920	10939	ALetter
10980	109B7	ALetter
109BE	109BF	ALetter
10A00		ALetter
10A01	10A03	Extend
10A05	10A06	Extend
10A0C	10A0F	Extend
10A10	10A13	ALetter
10A15	10A17	ALetter
10A19	10A35	ALetter
10A38	10A3A	Extend
10A3F		Extend
10A60	10A7C	ALetter
10A80	10A9C	ALetter
10AC0	10AC7	ALetter
10AC9	10AE4	ALetter
10AE5	10AE6	Extend
10B00	10B35	ALetter
10B40	10B55	ALetter
10B60	10B72	ALetter
10B80	10B91	ALetter
10C00	10C48	ALetter
10C80	10CB2	ALetter
10CC0	10CF2	ALetter
10D00	10D23	ALetter
10D24	10D27	Extend
10D30	10D39	Numeric
10D40	10D49	Numeric
10D4A	10D65	ALetter
10D69	10D6D	Extend
10D6F	10D85	ALetter
10E80	10EA9	ALetter
10EAB	10EAC	Extend
10EB0	10EB1	ALetter
10EC2	10EC4	ALetter
10EFC	10EFF	Extend
10F00	10F1C	ALetter
10F27		ALetter
10F30	10F45	ALetter
10F46	10F50	Extend
10F70	10F81	ALetter
10F82	10F85	Extend
10FB0	10FC4	ALetter
10FE0	10FF6	ALetter
11000	11002	Extend
11003	11037	ALetter
11038	11046	Extend
11066	1106F	Numeric
11070		Extend
11071	11072	ALetter
11073	11074	Extend
11075		ALetter
1107F	11082	Extend
11083	110AF	ALetter
110B0	110BA	Extend
110BD		Numeric
110C2		Extend
110CD		Numeric
110D0	110E8	ALetter
110F0	110F9	Numeric
11100	11102	Extend
11103	11126	ALetter
11127	11134	Extend
11136	1113F	Numeric
11144		ALetter
11145	11146	Extend
11147		ALetter
11150	11172	ALetter
11173		Extend
11176		ALetter
11180	11182	Extend
11183	111B2	ALetter
111B3	111C0	Extend
111C1	111C4	ALetter
111C9	111CC	Extend
111CE	111CF	Extend
111D0	111D9	Numeric
111DA		ALetter
111DC		ALetter
11200	11211	ALetter
11213	1122B	ALetter
1122C	11237	Extend
1123E		Extend
1123F	11240	ALetter
11241		Extend
11280	11286	ALetter
11288		ALetter
1128A	1128D	ALetter
1128F	1129D	ALetter
1129F	112A8	ALetter
112B0	112DE	ALetter
112DF	112EA	Extend
112F0	112F9	Numeric
11300	11303	Extend
11305	1130C	ALetter
1130F	11310	ALetter
11313	11328	ALetter
1132A	11330	ALetter
11332	11333	ALetter
11335	11339	ALetter
1133B	1133C	Extend
1133D		ALetter
1133E	11344	Extend
11347	11348	Extend
1134B	1134D	Extend
11350		ALetter
11357		Extend
1135D	11361	ALetter
11362	11363	Extend
11366	1136C	Extend
11370	11374	Extend
11380	11389	ALetter
1138B		ALetter
1138E		ALetter
11390	113B5	ALetter
113B7		ALetter
113B8	113C0	Extend
113C2		Extend
113C5		Extend
113C7	113CA	Extend
113CC	113D0	Extend
113D1		ALetter
113D2		Extend
113D3		ALetter
113E1	113E2	Extend
11400	11434	ALetter
11435	11446	Extend
11447	1144A	ALetter
11450	11459	Numeric
1145E		Extend
1145F	11461	ALetter
11480	114AF	ALetter
114B0	114C3	Extend
114C4	114C5	ALetter
114C7		ALetter
114D0	114D9	Numeric
11580	115AE	ALetter
115AF	115B5	Extend
115B8	115C0	Extend
115D8	115DB	ALetter
115DC	115DD	Extend
11600	1162F	ALetter
11630	11640	Extend
11644		ALetter
11650	11659	Numeric
11680	116AA	ALetter
116AB	116B7	Extend
116B8		ALetter
116C0	116C9	Numeric
116D0	116E3	Numeric
1171D	1172B	Extend
11730	11739	Numeric
11800	1182B	ALetter
1182C	1183A	Extend
118A0	118DF	ALetter
118E0	118E9	Numeric
118FF	11906	ALetter
11909		ALetter
1190C	11913	ALetter
11915	11916	ALetter
11918	1192F	ALetter
11930	11935	Extend
11937	11938	Extend
1193B	1193E	Extend
1193F		ALetter
11940		Extend
11941		ALetter
11942	11943	Extend
11950	11959	Numeric
119A0	119A7	ALetter
119AA	119D0	ALetter
119D1	119D7	Extend
119DA	119E0	Extend
119E1		ALetter
119E3		ALetter
119E4		Extend
11A00		ALetter
11A01	11A0A	Extend
11A0B	11A32	ALetter
11A33	11A39	Extend
11A3A		ALetter
11A3B	11A3E	Extend
11A47		Extend
11A50		ALetter
11A51	11A5B	Extend
11A5C	11A89	ALetter
11A8A	11A99	Extend
11A9D		ALetter
11AB0	11AF8	ALetter
11BC0	11BE0	ALetter
11BF0	11BF9	Numeric
11C00	11C08	ALetter
11C0A	11C2E	ALetter
11C2F	11C36	Extend
11C38	11C3F	Extend
11C40		ALetter
11C50	11C59	Numeric
11C72	11C8F	ALetter
11C92	11CA7	Extend
11CA9	11CB6	Extend
11D00	11D06	ALetter
11D08	11D09	ALetter
11D0B	11D30	ALetter
11D31	11D36	Extend
11D3A		Extend
11D3C	11D3D	Extend
11D3F	11D45	Extend
11D46		ALetter
11D47		Extend
11D50	11D59	Numeric
11D60	11D65	ALetter
11D67	11D68	ALetter
11D6A	11D89	ALetter
11D8A	11D8E	Extend
11D90	11D91	Extend
11D93	11D97	Extend
11D98		ALetter
11DA0	11DA9	Numeric
11EE0	11EF2	ALetter
11EF3	11EF6	Extend
11F00	11F01	Extend
11F02		ALetter
11F03		Extend
11F04	11F10	ALetter
11F12	11F33	ALetter
11F34	11F3A	Extend
11F3E	11F42	Extend
11F50	11F59	Numeric
11F5A		Extend
11FB0		ALetter
12000	12399	ALetter
12400	1246E	ALetter
12480	12543	ALetter
12F90	12FF0	ALetter
13000	1342F	ALetter
13430	1343F	Format
13440		Extend
13441	13446	ALetter
13447	13455	Extend
13460	143FA	ALetter
14400	14646	ALetter
16100	1611D	ALetter
1611E	1612F	Extend
16130	16139	Numeric
16800	16A38	ALetter
16A40	16A5E	ALetter
16A60	16A69	Numeric
16A70	16ABE	ALetter
16AC0	16AC9	Numeric
16AD0	16AED	ALetter
16AF0	16AF4	Extend
16B00	16B2F	ALetter
16B30	16B36	Extend
16B40	16B43	ALetter
16B50	16B59	Numeric
16B63	16B77	ALetter
16B7D	16B8F	ALetter
16D40	16D6C	ALetter
16D70	16D79	Numeric
16E40	16E7F	ALetter
16F00	16F4A	ALetter
16F4F		Extend
16F50		ALetter
16F51	16F87	Extend
16F8F	16F92	Extend
16F93	16F9F	ALetter
16FE0	16FE1	ALetter
16FE3		ALetter
16FE4		Extend
16FF0	16FF1	Extend
1AFF0	1AFF3	Katakana
1AFF5	1AFFB	Katakana
1AFFD	1AFFE	Katakana
1B000		Katakana
1B120	1B122	Katakana
1B155		Katakana
1B164	1B167	Katakana
1BC00	1BC6A	ALetter
1BC70	1BC7C	ALetter
1BC80	1BC88	ALetter
1BC90	1BC99	ALetter
1BC9D	1BC9E	Extend
1BCA0	1BCA3	Format
1CCF0	1CCF9	Numeric
1CF00	1CF2D	Extend
1CF30	1CF46	Extend
1D165	1D169	Extend
1D16D	1D172	Extend
1D173	1D17A	Format
1D17B	1D182	Extend
1D185	1D18B	Extend
1D1AA	1D1AD	Extend
1D242	1D244	Extend
1D400	1D454	ALetter
1D456	1D49C	ALetter
1D49E	1D49F	ALetter
1D4A2		ALetter
1D4A5	1D4A6	ALetter
1D4A9	1D4AC	ALetter
1D4AE	1D4B9	ALetter
1D4BB		ALetter
1D4BD	1D4C3	ALetter
1D4C5	1D505	ALetter
1D507	1D50A	ALetter
1D50D	1D514	ALetter
1D516	1D51C	ALetter
1D51E	1D539	ALetter
1D53B	1D53E	ALetter
1D540	1D544	ALetter
1D546		ALetter
1D54A	1D550	ALetter
1D552	1D6A5	ALetter
1D6A8	1D6C0	ALetter
1D6C2	1D6DA	ALetter
1D6DC	1D6FA	ALetter
1D6FC	1D714	ALetter
1D716	1D734	ALetter
1D736	1D74E	ALetter
1D750	1D76E	ALetter
1D770	1D788	ALetter
1D78A	1D7A8	ALetter
1D7AA	1D7C2	ALetter
1D7C4	1D7CB	ALetter
1D7CE	1D7FF	Numeric
1DA00	1DA36	Extend
1DA3B	1DA6C	Extend
1DA75		Extend
1DA84		Extend
1DA9B	1DA9F	Extend
1DAA1	1DAAF	Extend
1DF00	1DF1E	ALetter
1DF25	1DF2A	ALetter
1E000	1E006	Extend
1E008	1E018	Extend
1E01B	1E021	Extend
1E023	1E024	Extend
1E026	1E02A	Extend
1E030	1E06D	ALetter
1E08F		Extend
1E100	1E12C	ALetter
1E130	1E136	Extend
1E137	1E13D	ALetter
1E140	1E149	Numeric
1E14E		ALetter
1E290	1E2AD	ALetter
1E2AE		Extend
1E2C0	1E2EB	ALetter
1E2EC	1E2EF	Extend
1E2F0	1E2F9	Numeric
1E4D0	1E4EB	ALetter
1E4EC	1E4EF	Extend
1E4F0	1E4F9	Numeric
1E5D0	1E5ED	ALetter
1E5EE	1E5EF	Extend
1E5F0		ALetter
1E5F1	1E5FA	Numeric
1E7E0	1E7E6	ALetter
1E7E8	1E7EB	ALetter
1E7ED	1E7EE	ALetter
1E7F0	1E7FE	ALetter
1E800	1E8C4	ALetter
1E8D0	1E8D6	Extend
1E900	1E943	ALetter
1E944	1E94A	Extend
1E94B		ALetter
1E950	1E959	Numeric
1EE00	1EE03	ALetter
1EE05	1EE1F	ALetter
1EE21	1EE22	ALetter
1EE24		ALetter
1EE27		ALetter
1EE29	1EE32	ALetter
1EE34	1EE37	ALetter
1EE39		ALetter
1EE3B		ALetter
1EE42		ALetter
1EE47		ALetter
1EE49		ALetter
1EE4B		ALetter
1EE4D	1EE4F	ALetter
1EE51	1EE52	ALetter
1EE54		ALetter
1EE57		ALetter
1EE59		ALetter
1EE5B		ALetter
1EE5D		ALetter
1EE5F		ALetter
1EE61	1EE62	ALetter
1EE64		ALetter
1EE67	1EE6A	ALetter
1EE6C	1EE72	ALetter
1EE74	1EE77	ALetter
1EE79	1EE7C	ALetter
1EE7E		ALetter
1EE80	1EE89	ALetter
1EE8B	1EE9B	ALetter
1EEA1	1EEA3	ALetter
1EEA5	1EEA9	ALetter
1EEAB	1EEBB	ALetter
1F000	1F0FF	ExtPict_XX
1F10D	1F10F	ExtPict_XX
1F12F		ExtPict_XX
1F130	1F149	ALetter
1F150	1F169	ALetter
1F16C	1F16F	ExtPict_XX
1F170	1F171	ExtPict_LE
1F172	1F17D	ALetter
1F17E	1F17F	ExtPict_LE
1F180	1F189	ALetter
1F18E		ExtPict_XX
1F191	1F19A	ExtPict_XX
1F1AD	1F1E5	ExtPict_XX
1F1E6	1F1FF	Regional_Indicator
1F201	1F20F	ExtPict_XX
1F21A		ExtPict_XX
1F22F		ExtPict_XX
1F232	1F23A	ExtPict_XX
1F23C	1F23F	ExtPict_XX
1F249	1F3FA	ExtPict_XX
1F3FB	1F3FF	Extend
1F400	1F53D	ExtPict_XX
1F546	1F64F	ExtPict_XX
1F680	1F6FF	ExtPict_XX
1F774	1F77F	ExtPict_XX
1F7D5	1F7FF	ExtPict_XX
1F80C	1F80F	ExtPict_XX
1F848	1F84F	ExtPict_XX
1F85A	1F85F	ExtPict_XX
1F888	1F88F	ExtPict_XX
1F8AE	1F8FF	ExtPict_XX
1F90C	1F93A	ExtPict_XX
1F93C	1F945	ExtPict_XX
1F947	1FAFF	ExtPict_XX
1FBF0	1FBF9	Numeric
1FC00	1FFFD	ExtPict_XX
E0001		Format
E0020	E007F	Extend
E0100	E01EF	Extend
END
