/* Generated automatically by the program `gencodes'
   from the machine description file `md'.  */

#ifndef GCC_INSN_CODES_H
#define GCC_INSN_CODES_H

enum insn_code {
  CODE_FOR_nothing = 0,
  CODE_FOR_ls2_alu1_turn_enabled_insn = 1,
  CODE_FOR_ls2_alu2_turn_enabled_insn = 2,
  CODE_FOR_ls2_falu1_turn_enabled_insn = 3,
  CODE_FOR_ls2_falu2_turn_enabled_insn = 4,
  CODE_FOR_trap = 5,
  CODE_FOR_addsf3 = 10,
  CODE_FOR_adddf3 = 11,
  CODE_FOR_addv2sf3 = 12,
  CODE_FOR_subsf3 = 23,
  CODE_FOR_subdf3 = 24,
  CODE_FOR_subv2sf3 = 25,
  CODE_FOR_subsi3 = 26,
  CODE_FOR_subdi3 = 27,
  CODE_FOR_mulv2sf3 = 33,
  CODE_FOR_mulsi3_mul3_nohilo = 34,
  CODE_FOR_muldi3_mul3_nohilo = 35,
  CODE_FOR_mulsi3_mul3 = 36,
  CODE_FOR_muldi3_mul3 = 37,
  CODE_FOR_mulsi3_internal = 38,
  CODE_FOR_muldi3_internal = 39,
  CODE_FOR_mulsi3_r4000 = 40,
  CODE_FOR_muldi3_r4000 = 41,
  CODE_FOR_mulsidi3_32bit = 51,
  CODE_FOR_umulsidi3_32bit = 52,
  CODE_FOR_mulsidi3_32bit_r4000 = 53,
  CODE_FOR_umulsidi3_32bit_r4000 = 54,
  CODE_FOR_mulsidi3_64bit = 55,
  CODE_FOR_umulsidi3_64bit = 56,
  CODE_FOR_mulsidi3_64bit_hilo = 57,
  CODE_FOR_umulsidi3_64bit_hilo = 58,
  CODE_FOR_mulsidi3_64bit_dmul = 59,
  CODE_FOR_mulsidi3_64bit_r6dmul = 60,
  CODE_FOR_msubsidi4 = 63,
  CODE_FOR_umsubsidi4 = 64,
  CODE_FOR_smulsi3_highpart_r6 = 65,
  CODE_FOR_umulsi3_highpart_r6 = 66,
  CODE_FOR_smulsi3_highpart_internal = 67,
  CODE_FOR_umulsi3_highpart_internal = 68,
  CODE_FOR_smulsi3_highpart_mulhi_internal = 69,
  CODE_FOR_umulsi3_highpart_mulhi_internal = 70,
  CODE_FOR_smuldi3_highpart_r6 = 73,
  CODE_FOR_umuldi3_highpart_r6 = 74,
  CODE_FOR_smuldi3_highpart_internal = 75,
  CODE_FOR_umuldi3_highpart_internal = 76,
  CODE_FOR_mulditi3_internal = 77,
  CODE_FOR_umulditi3_internal = 78,
  CODE_FOR_mulditi3_r4000 = 79,
  CODE_FOR_umulditi3_r4000 = 80,
  CODE_FOR_madsi = 81,
  CODE_FOR_maddsidi4 = 82,
  CODE_FOR_umaddsidi4 = 83,
  CODE_FOR_divmodsi4_mips16 = 137,
  CODE_FOR_divmoddi4_mips16 = 138,
  CODE_FOR_udivmodsi4_mips16 = 141,
  CODE_FOR_udivmoddi4_mips16 = 142,
  CODE_FOR_divmodsi4_hilo_di = 143,
  CODE_FOR_udivmodsi4_hilo_di = 144,
   CODE_FOR_divmoddi4_hilo_di = CODE_FOR_nothing,
   CODE_FOR_udivmoddi4_hilo_di = CODE_FOR_nothing,
  CODE_FOR_divmodsi4_hilo_ti = 145,
  CODE_FOR_udivmodsi4_hilo_ti = 146,
  CODE_FOR_divmoddi4_hilo_ti = 147,
  CODE_FOR_udivmoddi4_hilo_ti = 148,
  CODE_FOR_divsi3 = 149,
  CODE_FOR_udivsi3 = 150,
  CODE_FOR_divdi3 = 151,
  CODE_FOR_udivdi3 = 152,
  CODE_FOR_modsi3 = 153,
  CODE_FOR_umodsi3 = 154,
  CODE_FOR_moddi3 = 155,
  CODE_FOR_umoddi3 = 156,
  CODE_FOR_sqrtsf2 = 157,
  CODE_FOR_sqrtdf2 = 158,
  CODE_FOR_sqrtv2sf2 = 159,
  CODE_FOR_abssf2 = 166,
  CODE_FOR_absdf2 = 167,
  CODE_FOR_absv2sf2 = 168,
  CODE_FOR_clzsi2 = 169,
  CODE_FOR_clzdi2 = 170,
  CODE_FOR_ctzsi2 = 173,
  CODE_FOR_ctzdi2 = 174,
  CODE_FOR_popcountsi2 = 175,
  CODE_FOR_popcountdi2 = 176,
  CODE_FOR_negsi2 = 178,
  CODE_FOR_negdi2 = 179,
  CODE_FOR_negsf2 = 180,
  CODE_FOR_negdf2 = 181,
  CODE_FOR_negv2sf2 = 182,
  CODE_FOR_one_cmplsi2 = 183,
  CODE_FOR_one_cmpldi2 = 184,
  CODE_FOR_truncdfsf2 = 200,
  CODE_FOR_truncdiqi2 = 201,
  CODE_FOR_truncdihi2 = 202,
  CODE_FOR_truncdisi2 = 203,
  CODE_FOR_extendsidi2 = 238,
  CODE_FOR_extendsfdf2 = 264,
  CODE_FOR_fix_truncdfsi2_insn = 265,
  CODE_FOR_fix_truncdfsi2_macro = 266,
  CODE_FOR_fix_truncsfsi2_insn = 267,
  CODE_FOR_fix_truncsfsi2_macro = 268,
  CODE_FOR_fix_truncdfdi2 = 269,
  CODE_FOR_fix_truncsfdi2 = 270,
  CODE_FOR_floatsidf2 = 271,
  CODE_FOR_floatdidf2 = 272,
  CODE_FOR_floatsisf2 = 273,
  CODE_FOR_floatdisf2 = 274,
  CODE_FOR_mov_lwl = 286,
  CODE_FOR_mov_ldl = 287,
  CODE_FOR_mov_lwr = 288,
  CODE_FOR_mov_ldr = 289,
  CODE_FOR_mov_swl = 290,
  CODE_FOR_mov_sdl = 291,
  CODE_FOR_mov_swr = 292,
  CODE_FOR_mov_sdr = 293,
  CODE_FOR_mov_swl_mips16e2 = 294,
  CODE_FOR_mov_sdl_mips16e2 = 295,
  CODE_FOR_mov_swr_mips16e2 = 296,
  CODE_FOR_mov_sdr_mips16e2 = 297,
  CODE_FOR_load_gotsi = 310,
  CODE_FOR_load_gotdi = 311,
  CODE_FOR_movccf = 375,
  CODE_FOR_mfhisi_di = 387,
   CODE_FOR_mfhidi_di = CODE_FOR_nothing,
  CODE_FOR_mfhisi_ti = 388,
  CODE_FOR_mfhidi_ti = 389,
  CODE_FOR_mthisi_di = 390,
   CODE_FOR_mthidi_di = CODE_FOR_nothing,
  CODE_FOR_mthisi_ti = 391,
  CODE_FOR_mthidi_ti = 392,
  CODE_FOR_load_lowdf = 393,
  CODE_FOR_load_lowdi = 394,
  CODE_FOR_load_lowv2sf = 395,
  CODE_FOR_load_lowv2si = 396,
  CODE_FOR_load_lowv4hi = 397,
  CODE_FOR_load_lowv8qi = 398,
  CODE_FOR_load_lowtf = 399,
  CODE_FOR_load_highdf = 400,
  CODE_FOR_load_highdi = 401,
  CODE_FOR_load_highv2sf = 402,
  CODE_FOR_load_highv2si = 403,
  CODE_FOR_load_highv4hi = 404,
  CODE_FOR_load_highv8qi = 405,
  CODE_FOR_load_hightf = 406,
  CODE_FOR_store_worddf = 407,
  CODE_FOR_store_worddi = 408,
  CODE_FOR_store_wordv2sf = 409,
  CODE_FOR_store_wordv2si = 410,
  CODE_FOR_store_wordv4hi = 411,
  CODE_FOR_store_wordv8qi = 412,
  CODE_FOR_store_wordtf = 413,
  CODE_FOR_mthc1df = 414,
  CODE_FOR_mthc1di = 415,
  CODE_FOR_mthc1v2sf = 416,
  CODE_FOR_mthc1v2si = 417,
  CODE_FOR_mthc1v4hi = 418,
  CODE_FOR_mthc1v8qi = 419,
  CODE_FOR_mthc1tf = 420,
  CODE_FOR_mfhc1df = 421,
  CODE_FOR_mfhc1di = 422,
  CODE_FOR_mfhc1v2sf = 423,
  CODE_FOR_mfhc1v2si = 424,
  CODE_FOR_mfhc1v4hi = 425,
  CODE_FOR_mfhc1v8qi = 426,
  CODE_FOR_mfhc1tf = 427,
  CODE_FOR_loadgp_newabi_si = 428,
  CODE_FOR_loadgp_newabi_di = 429,
  CODE_FOR_loadgp_absolute_si = 430,
  CODE_FOR_loadgp_absolute_di = 431,
  CODE_FOR_loadgp_blockage = 432,
  CODE_FOR_loadgp_rtp_si = 433,
  CODE_FOR_loadgp_rtp_di = 434,
  CODE_FOR_copygp_mips16_si = 435,
  CODE_FOR_copygp_mips16_di = 436,
  CODE_FOR_potential_cprestore_si = 437,
  CODE_FOR_potential_cprestore_di = 438,
  CODE_FOR_cprestore_si = 439,
  CODE_FOR_cprestore_di = 440,
  CODE_FOR_use_cprestore_si = 441,
  CODE_FOR_use_cprestore_di = 442,
  CODE_FOR_sync = 443,
  CODE_FOR_synci = 444,
  CODE_FOR_rdhwr_synci_step_si = 445,
  CODE_FOR_rdhwr_synci_step_di = 446,
  CODE_FOR_clear_hazard_si = 447,
  CODE_FOR_clear_hazard_di = 448,
  CODE_FOR_mips_cache = 449,
  CODE_FOR_r10k_cache_barrier = 450,
  CODE_FOR_lsa = 463,
  CODE_FOR_dlsa = 464,
  CODE_FOR_rotrsi3 = 469,
  CODE_FOR_rotrdi3 = 470,
  CODE_FOR_bswaphi2 = 471,
  CODE_FOR_bswapsi2 = 472,
  CODE_FOR_bswapdi2 = 473,
  CODE_FOR_wsbh = 474,
  CODE_FOR_dsbh = 475,
  CODE_FOR_dshd = 476,
  CODE_FOR_sunordered_sf_using_cc = 579,
  CODE_FOR_suneq_sf_using_cc = 580,
  CODE_FOR_sunlt_sf_using_cc = 581,
  CODE_FOR_sunle_sf_using_cc = 582,
  CODE_FOR_seq_sf_using_cc = 583,
  CODE_FOR_slt_sf_using_cc = 584,
  CODE_FOR_sle_sf_using_cc = 585,
   CODE_FOR_sordered_sf_using_cc = CODE_FOR_nothing,
   CODE_FOR_sltgt_sf_using_cc = CODE_FOR_nothing,
   CODE_FOR_sne_sf_using_cc = CODE_FOR_nothing,
  CODE_FOR_sunordered_df_using_cc = 586,
  CODE_FOR_suneq_df_using_cc = 587,
  CODE_FOR_sunlt_df_using_cc = 588,
  CODE_FOR_sunle_df_using_cc = 589,
  CODE_FOR_seq_df_using_cc = 590,
  CODE_FOR_slt_df_using_cc = 591,
  CODE_FOR_sle_df_using_cc = 592,
   CODE_FOR_sordered_df_using_cc = CODE_FOR_nothing,
   CODE_FOR_sltgt_df_using_cc = CODE_FOR_nothing,
   CODE_FOR_sne_df_using_cc = CODE_FOR_nothing,
  CODE_FOR_sunordered_sf_using_cce = 593,
  CODE_FOR_suneq_sf_using_cce = 594,
  CODE_FOR_sunlt_sf_using_cce = 595,
  CODE_FOR_sunle_sf_using_cce = 596,
  CODE_FOR_seq_sf_using_cce = 597,
  CODE_FOR_slt_sf_using_cce = 598,
  CODE_FOR_sle_sf_using_cce = 599,
   CODE_FOR_sordered_sf_using_cce = CODE_FOR_nothing,
   CODE_FOR_sltgt_sf_using_cce = CODE_FOR_nothing,
   CODE_FOR_sne_sf_using_cce = CODE_FOR_nothing,
  CODE_FOR_sunordered_df_using_cce = 600,
  CODE_FOR_suneq_df_using_cce = 601,
  CODE_FOR_sunlt_df_using_cce = 602,
  CODE_FOR_sunle_df_using_cce = 603,
  CODE_FOR_seq_df_using_cce = 604,
  CODE_FOR_slt_df_using_cce = 605,
  CODE_FOR_sle_df_using_cce = 606,
   CODE_FOR_sordered_df_using_cce = CODE_FOR_nothing,
   CODE_FOR_sltgt_df_using_cce = CODE_FOR_nothing,
   CODE_FOR_sne_df_using_cce = CODE_FOR_nothing,
  CODE_FOR_sunordered_sf_using_ccf = 607,
  CODE_FOR_suneq_sf_using_ccf = 608,
  CODE_FOR_sunlt_sf_using_ccf = 609,
  CODE_FOR_sunle_sf_using_ccf = 610,
  CODE_FOR_seq_sf_using_ccf = 611,
  CODE_FOR_slt_sf_using_ccf = 612,
  CODE_FOR_sle_sf_using_ccf = 613,
  CODE_FOR_sordered_sf_using_ccf = 614,
  CODE_FOR_sltgt_sf_using_ccf = 615,
  CODE_FOR_sne_sf_using_ccf = 616,
  CODE_FOR_sunordered_df_using_ccf = 617,
  CODE_FOR_suneq_df_using_ccf = 618,
  CODE_FOR_sunlt_df_using_ccf = 619,
  CODE_FOR_sunle_df_using_ccf = 620,
  CODE_FOR_seq_df_using_ccf = 621,
  CODE_FOR_slt_df_using_ccf = 622,
  CODE_FOR_sle_df_using_ccf = 623,
  CODE_FOR_sordered_df_using_ccf = 624,
  CODE_FOR_sltgt_df_using_ccf = 625,
  CODE_FOR_sne_df_using_ccf = 626,
  CODE_FOR_sge_sf_using_cc = 627,
  CODE_FOR_sgt_sf_using_cc = 628,
  CODE_FOR_sunge_sf_using_cc = 629,
  CODE_FOR_sungt_sf_using_cc = 630,
  CODE_FOR_sge_df_using_cc = 631,
  CODE_FOR_sgt_df_using_cc = 632,
  CODE_FOR_sunge_df_using_cc = 633,
  CODE_FOR_sungt_df_using_cc = 634,
  CODE_FOR_sge_sf_using_cce = 635,
  CODE_FOR_sgt_sf_using_cce = 636,
  CODE_FOR_sunge_sf_using_cce = 637,
  CODE_FOR_sungt_sf_using_cce = 638,
  CODE_FOR_sge_df_using_cce = 639,
  CODE_FOR_sgt_df_using_cce = 640,
  CODE_FOR_sunge_df_using_cce = 641,
  CODE_FOR_sungt_df_using_cce = 642,
  CODE_FOR_sge_sf_using_ccf = 643,
  CODE_FOR_sgt_sf_using_ccf = 644,
  CODE_FOR_sunge_sf_using_ccf = 645,
  CODE_FOR_sungt_sf_using_ccf = 646,
  CODE_FOR_sge_df_using_ccf = 647,
  CODE_FOR_sgt_df_using_ccf = 648,
  CODE_FOR_sunge_df_using_ccf = 649,
  CODE_FOR_sungt_df_using_ccf = 650,
  CODE_FOR_indirect_jump_si = 654,
  CODE_FOR_indirect_jump_di = 655,
  CODE_FOR_indirect_jump_and_restore_si = 656,
  CODE_FOR_indirect_jump_and_restore_di = 657,
  CODE_FOR_tablejump_si = 658,
  CODE_FOR_tablejump_di = 659,
  CODE_FOR_casesi_internal_mips16_si = 660,
  CODE_FOR_casesi_internal_mips16_di = 661,
  CODE_FOR_blockage = 662,
  CODE_FOR_probe_stack_range_si = 663,
  CODE_FOR_probe_stack_range_di = 664,
  CODE_FOR_mips_hb_return_internal = 667,
  CODE_FOR_return_internal = 668,
  CODE_FOR_simple_return_internal = 669,
  CODE_FOR_mips_eret = 670,
  CODE_FOR_mips_deret = 671,
  CODE_FOR_mips_di = 672,
  CODE_FOR_mips_ehb = 673,
  CODE_FOR_mips_rdpgpr_si = 674,
  CODE_FOR_mips_rdpgpr_di = 675,
  CODE_FOR_cop0_move = 676,
  CODE_FOR_eh_set_lr_si = 677,
  CODE_FOR_eh_set_lr_di = 678,
  CODE_FOR_restore_gp_si = 679,
  CODE_FOR_restore_gp_di = 680,
  CODE_FOR_move_gpsi = 681,
  CODE_FOR_move_gpdi = 682,
  CODE_FOR_load_callsi = 683,
  CODE_FOR_load_calldi = 684,
  CODE_FOR_set_got_version = 685,
  CODE_FOR_update_got_version = 686,
  CODE_FOR_sibcall_internal = 687,
  CODE_FOR_sibcall_value_internal = 688,
  CODE_FOR_sibcall_value_multiple_internal = 689,
  CODE_FOR_call_internal = 690,
  CODE_FOR_call_split = 691,
  CODE_FOR_call_internal_direct = 692,
  CODE_FOR_call_direct_split = 693,
  CODE_FOR_call_value_internal = 694,
  CODE_FOR_call_value_split = 695,
  CODE_FOR_call_value_internal_direct = 696,
  CODE_FOR_call_value_direct_split = 697,
  CODE_FOR_call_value_multiple_internal = 698,
  CODE_FOR_call_value_multiple_split = 699,
  CODE_FOR_prefetch = 700,
  CODE_FOR_nop = 703,
  CODE_FOR_hazard_nop = 704,
  CODE_FOR_insn_pseudo = 705,
  CODE_FOR_movsi_on_si = 706,
  CODE_FOR_movdi_on_si = 707,
  CODE_FOR_movsi_on_di = 708,
  CODE_FOR_movdi_on_di = 709,
  CODE_FOR_movsi_on_cc = 710,
  CODE_FOR_movdi_on_cc = 711,
  CODE_FOR_movsi_on_cce = 712,
  CODE_FOR_movdi_on_cce = 713,
  CODE_FOR_movsi_on_si_mips16e2 = 714,
  CODE_FOR_movdi_on_si_mips16e2 = 715,
  CODE_FOR_movsi_on_di_mips16e2 = 716,
  CODE_FOR_movdi_on_di_mips16e2 = 717,
  CODE_FOR_movsi_on_cc_mips16e2 = 718,
  CODE_FOR_movdi_on_cc_mips16e2 = 719,
  CODE_FOR_movsi_on_cce_mips16e2 = 720,
  CODE_FOR_movdi_on_cce_mips16e2 = 721,
  CODE_FOR_movsi_on_si_ne = 722,
  CODE_FOR_movdi_on_si_ne = 723,
  CODE_FOR_movsi_on_di_ne = 724,
  CODE_FOR_movdi_on_di_ne = 725,
  CODE_FOR_movsi_on_si_ne_mips16e2 = 726,
  CODE_FOR_movdi_on_si_ne_mips16e2 = 727,
  CODE_FOR_movsi_on_di_ne_mips16e2 = 728,
  CODE_FOR_movdi_on_di_ne_mips16e2 = 729,
  CODE_FOR_consttable = 748,
  CODE_FOR_consttable_end = 749,
  CODE_FOR_consttable_tls_reloc = 750,
  CODE_FOR_consttable_int = 751,
  CODE_FOR_consttable_float = 752,
  CODE_FOR_align = 753,
  CODE_FOR_tls_get_tp_si = 755,
  CODE_FOR_tls_get_tp_di = 756,
  CODE_FOR_tls_get_tp_mips16_si = 759,
  CODE_FOR_tls_get_tp_mips16_di = 760,
  CODE_FOR_mips_get_fcsr_mips16_si = 764,
  CODE_FOR_mips_get_fcsr_mips16_di = 765,
  CODE_FOR_mips_set_fcsr_mips16_si = 767,
  CODE_FOR_mips_set_fcsr_mips16_di = 768,
  CODE_FOR_sminsf3 = 775,
  CODE_FOR_smindf3 = 776,
  CODE_FOR_smaxsf3 = 777,
  CODE_FOR_smaxdf3 = 778,
  CODE_FOR_fminsf3 = 779,
  CODE_FOR_fmindf3 = 780,
  CODE_FOR_fmaxsf3 = 781,
  CODE_FOR_fmaxdf3 = 782,
  CODE_FOR_fmin_a_sf = 783,
  CODE_FOR_fmin_a_df = 784,
  CODE_FOR_fmax_a_sf = 785,
  CODE_FOR_fmax_a_df = 786,
  CODE_FOR_rintsf2 = 787,
  CODE_FOR_rintdf2 = 788,
  CODE_FOR_fclass_sf = 789,
  CODE_FOR_fclass_df = 790,
  CODE_FOR_sync_compare_and_swapsi = 792,
  CODE_FOR_sync_compare_and_swapdi = 793,
  CODE_FOR_compare_and_swap_12 = 794,
  CODE_FOR_sync_addsi = 795,
  CODE_FOR_sync_adddi = 796,
  CODE_FOR_sync_add_12 = 797,
  CODE_FOR_sync_sub_12 = 798,
  CODE_FOR_sync_ior_12 = 799,
  CODE_FOR_sync_xor_12 = 800,
  CODE_FOR_sync_and_12 = 801,
  CODE_FOR_sync_old_add_12 = 802,
  CODE_FOR_sync_old_sub_12 = 803,
  CODE_FOR_sync_old_ior_12 = 804,
  CODE_FOR_sync_old_xor_12 = 805,
  CODE_FOR_sync_old_and_12 = 806,
  CODE_FOR_sync_new_add_12 = 807,
  CODE_FOR_sync_new_sub_12 = 808,
  CODE_FOR_sync_new_ior_12 = 809,
  CODE_FOR_sync_new_xor_12 = 810,
  CODE_FOR_sync_new_and_12 = 811,
  CODE_FOR_sync_nand_12 = 812,
  CODE_FOR_sync_old_nand_12 = 813,
  CODE_FOR_sync_new_nand_12 = 814,
  CODE_FOR_sync_subsi = 815,
  CODE_FOR_sync_subdi = 816,
  CODE_FOR_sync_old_addsi = 817,
  CODE_FOR_sync_old_adddi = 818,
  CODE_FOR_sync_old_subsi = 819,
  CODE_FOR_sync_old_subdi = 820,
  CODE_FOR_sync_new_addsi = 821,
  CODE_FOR_sync_new_adddi = 822,
  CODE_FOR_sync_new_subsi = 823,
  CODE_FOR_sync_new_subdi = 824,
  CODE_FOR_sync_iorsi = 825,
  CODE_FOR_sync_xorsi = 826,
  CODE_FOR_sync_andsi = 827,
  CODE_FOR_sync_iordi = 828,
  CODE_FOR_sync_xordi = 829,
  CODE_FOR_sync_anddi = 830,
  CODE_FOR_sync_old_iorsi = 831,
  CODE_FOR_sync_old_xorsi = 832,
  CODE_FOR_sync_old_andsi = 833,
  CODE_FOR_sync_old_iordi = 834,
  CODE_FOR_sync_old_xordi = 835,
  CODE_FOR_sync_old_anddi = 836,
  CODE_FOR_sync_new_iorsi = 837,
  CODE_FOR_sync_new_xorsi = 838,
  CODE_FOR_sync_new_andsi = 839,
  CODE_FOR_sync_new_iordi = 840,
  CODE_FOR_sync_new_xordi = 841,
  CODE_FOR_sync_new_anddi = 842,
  CODE_FOR_sync_nandsi = 843,
  CODE_FOR_sync_nanddi = 844,
  CODE_FOR_sync_old_nandsi = 845,
  CODE_FOR_sync_old_nanddi = 846,
  CODE_FOR_sync_new_nandsi = 847,
  CODE_FOR_sync_new_nanddi = 848,
  CODE_FOR_sync_lock_test_and_setsi = 849,
  CODE_FOR_sync_lock_test_and_setdi = 850,
  CODE_FOR_test_and_set_12 = 851,
  CODE_FOR_atomic_compare_and_swapsi = 852,
  CODE_FOR_atomic_compare_and_swapdi = 853,
  CODE_FOR_atomic_exchangesi_llsc = 854,
  CODE_FOR_atomic_exchangedi_llsc = 855,
  CODE_FOR_atomic_exchangesi_swap = 856,
  CODE_FOR_atomic_exchangedi_swap = 857,
  CODE_FOR_atomic_fetch_addsi_llsc = 858,
  CODE_FOR_atomic_fetch_adddi_llsc = 859,
  CODE_FOR_atomic_fetch_addsi_ldadd = 860,
  CODE_FOR_atomic_fetch_adddi_ldadd = 861,
  CODE_FOR_mips_cond_move_tf_ps = 864,
  CODE_FOR_vec_perm_const_ps = 865,
  CODE_FOR_vec_concatv2sf = 866,
  CODE_FOR_vec_extractv2sfsf = 867,
  CODE_FOR_mips_alnv_ps = 868,
  CODE_FOR_mips_addr_ps = 869,
  CODE_FOR_mips_cvt_pw_ps = 870,
  CODE_FOR_mips_cvt_ps_pw = 871,
  CODE_FOR_mips_mulr_ps = 872,
  CODE_FOR_mips_cabs_cond_s = 874,
  CODE_FOR_mips_cabs_cond_d = 875,
  CODE_FOR_mips_c_cond_4s = 876,
  CODE_FOR_mips_cabs_cond_4s = 877,
  CODE_FOR_mips_c_cond_ps = 878,
  CODE_FOR_mips_cabs_cond_ps = 879,
  CODE_FOR_sunordered_ps = 880,
  CODE_FOR_suneq_ps = 881,
  CODE_FOR_sunlt_ps = 882,
  CODE_FOR_sunle_ps = 883,
  CODE_FOR_seq_ps = 884,
  CODE_FOR_slt_ps = 885,
  CODE_FOR_sle_ps = 886,
  CODE_FOR_sordered_ps = 887,
  CODE_FOR_sltgt_ps = 888,
  CODE_FOR_sne_ps = 889,
  CODE_FOR_sge_ps = 890,
  CODE_FOR_sgt_ps = 891,
  CODE_FOR_sunge_ps = 892,
  CODE_FOR_sungt_ps = 893,
  CODE_FOR_bc1any4t = 894,
  CODE_FOR_bc1any4f = 895,
  CODE_FOR_bc1any2t = 896,
  CODE_FOR_bc1any2f = 897,
  CODE_FOR_mips_rsqrt1_s = 900,
  CODE_FOR_mips_rsqrt1_d = 901,
  CODE_FOR_mips_rsqrt1_ps = 902,
  CODE_FOR_mips_rsqrt2_s = 903,
  CODE_FOR_mips_rsqrt2_d = 904,
  CODE_FOR_mips_rsqrt2_ps = 905,
  CODE_FOR_mips_recip1_s = 906,
  CODE_FOR_mips_recip1_d = 907,
  CODE_FOR_mips_recip1_ps = 908,
  CODE_FOR_mips_recip2_s = 909,
  CODE_FOR_mips_recip2_d = 910,
  CODE_FOR_mips_recip2_ps = 911,
  CODE_FOR_addv2hi3 = 912,
  CODE_FOR_addv4qi3 = 913,
  CODE_FOR_mips_addq_s_w = 914,
  CODE_FOR_mips_addq_s_ph = 915,
  CODE_FOR_mips_addu_s_qb = 916,
  CODE_FOR_subv2hi3 = 917,
  CODE_FOR_subv4qi3 = 918,
  CODE_FOR_mips_subq_s_w = 919,
  CODE_FOR_mips_subq_s_ph = 920,
  CODE_FOR_mips_subu_s_qb = 921,
  CODE_FOR_mips_addsc = 922,
  CODE_FOR_mips_addwc = 923,
  CODE_FOR_mips_modsub = 924,
  CODE_FOR_mips_raddu_w_qb = 925,
  CODE_FOR_mips_absq_s_w = 926,
  CODE_FOR_mips_absq_s_ph = 927,
  CODE_FOR_mips_precrq_qb_ph = 928,
  CODE_FOR_mips_precrq_ph_w = 929,
  CODE_FOR_mips_precrq_rs_ph_w = 930,
  CODE_FOR_mips_precrqu_s_qb_ph = 931,
  CODE_FOR_mips_preceq_w_phl = 932,
  CODE_FOR_mips_preceq_w_phr = 933,
  CODE_FOR_mips_precequ_ph_qbl = 934,
  CODE_FOR_mips_precequ_ph_qbr = 935,
  CODE_FOR_mips_precequ_ph_qbla = 936,
  CODE_FOR_mips_precequ_ph_qbra = 937,
  CODE_FOR_mips_preceu_ph_qbl = 938,
  CODE_FOR_mips_preceu_ph_qbr = 939,
  CODE_FOR_mips_preceu_ph_qbla = 940,
  CODE_FOR_mips_preceu_ph_qbra = 941,
  CODE_FOR_mips_shll_ph = 942,
  CODE_FOR_mips_shll_qb = 943,
  CODE_FOR_mips_shll_s_w = 944,
  CODE_FOR_mips_shll_s_ph = 945,
  CODE_FOR_mips_shrl_qb = 946,
  CODE_FOR_mips_shra_ph = 947,
  CODE_FOR_mips_shra_r_w = 948,
  CODE_FOR_mips_shra_r_ph = 949,
  CODE_FOR_mips_muleu_s_ph_qbl = 950,
  CODE_FOR_mips_muleu_s_ph_qbr = 951,
  CODE_FOR_mips_mulq_rs_ph = 952,
  CODE_FOR_mips_muleq_s_w_phl = 953,
  CODE_FOR_mips_muleq_s_w_phr = 954,
  CODE_FOR_mips_dpau_h_qbl = 955,
  CODE_FOR_mips_dpau_h_qbr = 956,
  CODE_FOR_mips_dpsu_h_qbl = 957,
  CODE_FOR_mips_dpsu_h_qbr = 958,
  CODE_FOR_mips_dpaq_s_w_ph = 959,
  CODE_FOR_mips_dpsq_s_w_ph = 960,
  CODE_FOR_mips_mulsaq_s_w_ph = 961,
  CODE_FOR_mips_dpaq_sa_l_w = 962,
  CODE_FOR_mips_dpsq_sa_l_w = 963,
  CODE_FOR_mips_maq_s_w_phl = 964,
  CODE_FOR_mips_maq_s_w_phr = 965,
  CODE_FOR_mips_maq_sa_w_phl = 966,
  CODE_FOR_mips_maq_sa_w_phr = 967,
  CODE_FOR_mips_bitrev = 968,
  CODE_FOR_mips_insv = 969,
  CODE_FOR_mips_repl_qb = 970,
  CODE_FOR_mips_repl_ph = 971,
  CODE_FOR_mips_cmp_eq_ph = 972,
  CODE_FOR_mips_cmpu_eq_qb = 973,
  CODE_FOR_mips_cmp_lt_ph = 974,
  CODE_FOR_mips_cmpu_lt_qb = 975,
  CODE_FOR_mips_cmp_le_ph = 976,
  CODE_FOR_mips_cmpu_le_qb = 977,
  CODE_FOR_mips_cmpgu_eq_qb = 978,
  CODE_FOR_mips_cmpgu_lt_qb = 979,
  CODE_FOR_mips_cmpgu_le_qb = 980,
  CODE_FOR_mips_pick_ph = 981,
  CODE_FOR_mips_pick_qb = 982,
  CODE_FOR_mips_packrl_ph = 983,
  CODE_FOR_mips_extr_w = 984,
  CODE_FOR_mips_extr_r_w = 985,
  CODE_FOR_mips_extr_rs_w = 986,
  CODE_FOR_mips_extr_s_h = 987,
  CODE_FOR_mips_extp = 988,
  CODE_FOR_mips_extpdp = 989,
  CODE_FOR_mips_shilo = 990,
  CODE_FOR_mips_mthlip = 991,
  CODE_FOR_mips_wrdsp = 992,
  CODE_FOR_mips_rddsp = 993,
  CODE_FOR_mips_lbx_extsi_si = 994,
  CODE_FOR_mips_lbux_extsi_si = 995,
  CODE_FOR_mips_lbx_extdi_si = 996,
  CODE_FOR_mips_lbux_extdi_si = 997,
  CODE_FOR_mips_lhx_extsi_si = 998,
  CODE_FOR_mips_lhux_extsi_si = 999,
  CODE_FOR_mips_lhx_extdi_si = 1000,
  CODE_FOR_mips_lhux_extdi_si = 1001,
  CODE_FOR_mips_lbx_extsi_di = 1002,
  CODE_FOR_mips_lbux_extsi_di = 1003,
  CODE_FOR_mips_lbx_extdi_di = 1004,
  CODE_FOR_mips_lbux_extdi_di = 1005,
  CODE_FOR_mips_lhx_extsi_di = 1006,
  CODE_FOR_mips_lhux_extsi_di = 1007,
  CODE_FOR_mips_lhx_extdi_di = 1008,
  CODE_FOR_mips_lhux_extdi_di = 1009,
  CODE_FOR_mips_lwx_si = 1010,
  CODE_FOR_mips_ldx_si = 1011,
  CODE_FOR_mips_lwx_di = 1012,
  CODE_FOR_mips_ldx_di = 1013,
  CODE_FOR_mips_bposge = 1018,
  CODE_FOR_mips_absq_s_qb = 1019,
  CODE_FOR_mips_addu_ph = 1020,
  CODE_FOR_mips_addu_s_ph = 1021,
  CODE_FOR_mips_adduh_qb = 1022,
  CODE_FOR_mips_adduh_r_qb = 1023,
  CODE_FOR_mips_append = 1024,
  CODE_FOR_mips_balign = 1025,
  CODE_FOR_mips_cmpgdu_eq_qb = 1026,
  CODE_FOR_mips_cmpgdu_lt_qb = 1027,
  CODE_FOR_mips_cmpgdu_le_qb = 1028,
  CODE_FOR_mips_dpa_w_ph = 1029,
  CODE_FOR_mips_dps_w_ph = 1030,
  CODE_FOR_mulv2hi3 = 1031,
  CODE_FOR_mips_mul_s_ph = 1032,
  CODE_FOR_mips_mulq_rs_w = 1033,
  CODE_FOR_mips_mulq_s_ph = 1034,
  CODE_FOR_mips_mulq_s_w = 1035,
  CODE_FOR_mips_mulsa_w_ph = 1036,
  CODE_FOR_mips_precr_qb_ph = 1037,
  CODE_FOR_mips_precr_sra_ph_w = 1038,
  CODE_FOR_mips_precr_sra_r_ph_w = 1039,
  CODE_FOR_mips_prepend = 1040,
  CODE_FOR_mips_shra_qb = 1041,
  CODE_FOR_mips_shra_r_qb = 1042,
  CODE_FOR_mips_shrl_ph = 1043,
  CODE_FOR_mips_subu_ph = 1044,
  CODE_FOR_mips_subu_s_ph = 1045,
  CODE_FOR_mips_subuh_qb = 1046,
  CODE_FOR_mips_subuh_r_qb = 1047,
  CODE_FOR_mips_addqh_ph = 1048,
  CODE_FOR_mips_addqh_r_ph = 1049,
  CODE_FOR_mips_addqh_w = 1050,
  CODE_FOR_mips_addqh_r_w = 1051,
  CODE_FOR_mips_subqh_ph = 1052,
  CODE_FOR_mips_subqh_r_ph = 1053,
  CODE_FOR_mips_subqh_w = 1054,
  CODE_FOR_mips_subqh_r_w = 1055,
  CODE_FOR_mips_dpax_w_ph = 1056,
  CODE_FOR_mips_dpsx_w_ph = 1057,
  CODE_FOR_mips_dpaqx_s_w_ph = 1058,
  CODE_FOR_mips_dpaqx_sa_w_ph = 1059,
  CODE_FOR_mips_dpsqx_s_w_ph = 1060,
  CODE_FOR_mips_dpsqx_sa_w_ph = 1061,
  CODE_FOR_addqq3 = 1062,
  CODE_FOR_addhq3 = 1063,
  CODE_FOR_addsq3 = 1064,
  CODE_FOR_adddq3 = 1065,
  CODE_FOR_adduqq3 = 1066,
  CODE_FOR_adduhq3 = 1067,
  CODE_FOR_addusq3 = 1068,
  CODE_FOR_addudq3 = 1069,
  CODE_FOR_addha3 = 1070,
  CODE_FOR_addsa3 = 1071,
  CODE_FOR_addda3 = 1072,
  CODE_FOR_adduha3 = 1073,
  CODE_FOR_addusa3 = 1074,
  CODE_FOR_adduda3 = 1075,
  CODE_FOR_usadduqq3 = 1076,
  CODE_FOR_usadduhq3 = 1077,
  CODE_FOR_usadduha3 = 1078,
  CODE_FOR_usaddv4uqq3 = 1079,
  CODE_FOR_usaddv2uhq3 = 1080,
  CODE_FOR_usaddv2uha3 = 1081,
  CODE_FOR_ssaddhq3 = 1082,
  CODE_FOR_ssaddsq3 = 1083,
  CODE_FOR_ssaddha3 = 1084,
  CODE_FOR_ssaddsa3 = 1085,
  CODE_FOR_ssaddv2hq3 = 1086,
  CODE_FOR_ssaddv2ha3 = 1087,
  CODE_FOR_subqq3 = 1088,
  CODE_FOR_subhq3 = 1089,
  CODE_FOR_subsq3 = 1090,
  CODE_FOR_subdq3 = 1091,
  CODE_FOR_subuqq3 = 1092,
  CODE_FOR_subuhq3 = 1093,
  CODE_FOR_subusq3 = 1094,
  CODE_FOR_subudq3 = 1095,
  CODE_FOR_subha3 = 1096,
  CODE_FOR_subsa3 = 1097,
  CODE_FOR_subda3 = 1098,
  CODE_FOR_subuha3 = 1099,
  CODE_FOR_subusa3 = 1100,
  CODE_FOR_subuda3 = 1101,
  CODE_FOR_ussubuqq3 = 1102,
  CODE_FOR_ussubuhq3 = 1103,
  CODE_FOR_ussubuha3 = 1104,
  CODE_FOR_ussubv4uqq3 = 1105,
  CODE_FOR_ussubv2uhq3 = 1106,
  CODE_FOR_ussubv2uha3 = 1107,
  CODE_FOR_sssubhq3 = 1108,
  CODE_FOR_sssubsq3 = 1109,
  CODE_FOR_sssubha3 = 1110,
  CODE_FOR_sssubsa3 = 1111,
  CODE_FOR_sssubv2hq3 = 1112,
  CODE_FOR_sssubv2ha3 = 1113,
  CODE_FOR_ssmulv2hq3 = 1114,
  CODE_FOR_ssmulhq3 = 1115,
  CODE_FOR_ssmulsq3 = 1116,
  CODE_FOR_ssmaddsqdq4 = 1117,
  CODE_FOR_ssmsubsqdq4 = 1118,
  CODE_FOR_jraddiusp = 1123,
  CODE_FOR_movv2si_internal = 1128,
  CODE_FOR_movv4hi_internal = 1129,
  CODE_FOR_movv8qi_internal = 1130,
  CODE_FOR_loongson_vec_init1_v4hi = 1131,
  CODE_FOR_loongson_vec_init1_v8qi = 1132,
  CODE_FOR_vec_pack_ssat_v2si = 1134,
  CODE_FOR_vec_pack_ssat_v4hi = 1135,
  CODE_FOR_vec_pack_usat_v4hi = 1136,
  CODE_FOR_addv2si3 = 1137,
  CODE_FOR_addv4hi3 = 1138,
  CODE_FOR_addv8qi3 = 1139,
  CODE_FOR_loongson_paddd = 1140,
  CODE_FOR_ssaddv4hi3 = 1141,
  CODE_FOR_ssaddv8qi3 = 1142,
  CODE_FOR_usaddv4hi3 = 1143,
  CODE_FOR_usaddv8qi3 = 1144,
  CODE_FOR_loongson_pandn_w = 1145,
  CODE_FOR_loongson_pandn_h = 1146,
  CODE_FOR_loongson_pandn_b = 1147,
  CODE_FOR_loongson_pandn_d = 1148,
  CODE_FOR_andv2si3 = 1149,
  CODE_FOR_andv4hi3 = 1150,
  CODE_FOR_andv8qi3 = 1151,
  CODE_FOR_iorv2si3 = 1152,
  CODE_FOR_iorv4hi3 = 1153,
  CODE_FOR_iorv8qi3 = 1154,
  CODE_FOR_xorv2si3 = 1155,
  CODE_FOR_xorv4hi3 = 1156,
  CODE_FOR_xorv8qi3 = 1157,
  CODE_FOR_one_cmplv2si2 = 1161,
  CODE_FOR_one_cmplv4hi2 = 1162,
  CODE_FOR_one_cmplv8qi2 = 1163,
  CODE_FOR_loongson_pavgh = 1164,
  CODE_FOR_loongson_pavgb = 1165,
  CODE_FOR_loongson_pcmpeqw = 1166,
  CODE_FOR_loongson_pcmpeqh = 1167,
  CODE_FOR_loongson_pcmpeqb = 1168,
  CODE_FOR_loongson_pcmpgtw = 1169,
  CODE_FOR_loongson_pcmpgth = 1170,
  CODE_FOR_loongson_pcmpgtb = 1171,
  CODE_FOR_loongson_pextrh = 1172,
  CODE_FOR_loongson_pinsrh_0 = 1173,
  CODE_FOR_loongson_pinsrh_1 = 1174,
  CODE_FOR_loongson_pinsrh_2 = 1175,
  CODE_FOR_loongson_pinsrh_3 = 1176,
  CODE_FOR_loongson_pmaddhw = 1178,
  CODE_FOR_smaxv4hi3 = 1179,
  CODE_FOR_umaxv8qi3 = 1180,
  CODE_FOR_sminv4hi3 = 1181,
  CODE_FOR_uminv8qi3 = 1182,
  CODE_FOR_loongson_pmovmskb = 1183,
  CODE_FOR_umulv4hi3_highpart = 1184,
  CODE_FOR_smulv4hi3_highpart = 1185,
  CODE_FOR_mulv4hi3 = 1186,
  CODE_FOR_loongson_pmuluw = 1187,
  CODE_FOR_loongson_pasubub = 1188,
  CODE_FOR_loongson_biadd = 1189,
  CODE_FOR_reduc_uplus_v8qi = 1190,
  CODE_FOR_loongson_psadbh = 1191,
  CODE_FOR_loongson_pshufh = 1192,
  CODE_FOR_ashlv2si3 = 1193,
  CODE_FOR_ashlv4hi3 = 1194,
  CODE_FOR_ashrv2si3 = 1195,
  CODE_FOR_ashrv4hi3 = 1196,
  CODE_FOR_lshrv2si3 = 1197,
  CODE_FOR_lshrv4hi3 = 1198,
  CODE_FOR_subv2si3 = 1199,
  CODE_FOR_subv4hi3 = 1200,
  CODE_FOR_subv8qi3 = 1201,
  CODE_FOR_loongson_psubd = 1202,
  CODE_FOR_sssubv4hi3 = 1203,
  CODE_FOR_sssubv8qi3 = 1204,
  CODE_FOR_ussubv4hi3 = 1205,
  CODE_FOR_ussubv8qi3 = 1206,
  CODE_FOR_loongson_punpckhbh = 1207,
  CODE_FOR_loongson_punpckhhw = 1208,
  CODE_FOR_loongson_punpckhhw_qi = 1209,
  CODE_FOR_loongson_punpckhwd = 1210,
  CODE_FOR_loongson_punpckhwd_qi = 1211,
  CODE_FOR_loongson_punpckhwd_hi = 1212,
  CODE_FOR_loongson_punpcklbh = 1213,
  CODE_FOR_loongson_punpcklhw = 1214,
  CODE_FOR_loongson_punpcklwd = 1216,
  CODE_FOR_vec_shl_v2si = 1219,
  CODE_FOR_vec_shl_v4hi = 1220,
  CODE_FOR_vec_shl_v8qi = 1221,
  CODE_FOR_vec_shl_di = 1222,
  CODE_FOR_vec_shr_v2si = 1223,
  CODE_FOR_vec_shr_v4hi = 1224,
  CODE_FOR_vec_shr_v8qi = 1225,
  CODE_FOR_vec_shr_di = 1226,
  CODE_FOR_vec_loongson_extract_lo_v2si = 1227,
  CODE_FOR_vec_loongson_extract_lo_v4hi = 1228,
  CODE_FOR_vec_loongson_extract_lo_v8qi = 1229,
  CODE_FOR_vec_pack_trunc_v2di = 1230,
  CODE_FOR_vec_pack_trunc_v4si = 1231,
  CODE_FOR_vec_pack_trunc_v8hi = 1232,
  CODE_FOR_msa_vec_extract_d_f = 1233,
  CODE_FOR_msa_vec_extract_w_f = 1234,
  CODE_FOR_msa_insert_d_f = 1235,
  CODE_FOR_msa_insert_w_f = 1236,
  CODE_FOR_msa_insert_d = 1237,
  CODE_FOR_msa_insert_w = 1238,
  CODE_FOR_msa_insert_h = 1239,
  CODE_FOR_msa_insert_b = 1240,
  CODE_FOR_msa_insve_d_f = 1241,
  CODE_FOR_msa_insve_w_f = 1242,
  CODE_FOR_msa_insve_d = 1243,
  CODE_FOR_msa_insve_w = 1244,
  CODE_FOR_msa_insve_h = 1245,
  CODE_FOR_msa_insve_b = 1246,
  CODE_FOR_msa_insve_d_f_scalar = 1247,
  CODE_FOR_msa_insve_w_f_scalar = 1248,
  CODE_FOR_msa_copy_s_h = 1249,
  CODE_FOR_msa_copy_u_h = 1250,
  CODE_FOR_msa_copy_s_b = 1251,
  CODE_FOR_msa_copy_u_b = 1252,
  CODE_FOR_msa_copy_u_w = 1253,
  CODE_FOR_msa_copy_s_w_64bit = 1254,
  CODE_FOR_msa_copy_s_w_f_64bit = 1255,
  CODE_FOR_msa_copy_s_w = 1256,
  CODE_FOR_msa_copy_s_w_f = 1257,
  CODE_FOR_msa_copy_s_d = 1258,
  CODE_FOR_msa_copy_s_d_f = 1259,
  CODE_FOR_negv2df2 = 1260,
  CODE_FOR_negv4sf2 = 1261,
  CODE_FOR_vec_permv2df = 1262,
  CODE_FOR_vec_permv4sf = 1263,
  CODE_FOR_vec_permv2di = 1264,
  CODE_FOR_vec_permv4si = 1265,
  CODE_FOR_vec_permv8hi = 1266,
  CODE_FOR_vec_permv16qi = 1267,
  CODE_FOR_movv2df_msa = 1268,
  CODE_FOR_movv4sf_msa = 1269,
  CODE_FOR_movv2di_msa = 1270,
  CODE_FOR_movv4si_msa = 1271,
  CODE_FOR_movv8hi_msa = 1272,
  CODE_FOR_movv16qi_msa = 1273,
  CODE_FOR_addv2di3 = 1274,
  CODE_FOR_addv4si3 = 1275,
  CODE_FOR_addv8hi3 = 1276,
  CODE_FOR_addv16qi3 = 1277,
  CODE_FOR_subv2di3 = 1278,
  CODE_FOR_subv4si3 = 1279,
  CODE_FOR_subv8hi3 = 1280,
  CODE_FOR_subv16qi3 = 1281,
  CODE_FOR_mulv2di3 = 1282,
  CODE_FOR_mulv4si3 = 1283,
  CODE_FOR_mulv8hi3 = 1284,
  CODE_FOR_mulv16qi3 = 1285,
  CODE_FOR_msa_maddv_d = 1286,
  CODE_FOR_msa_maddv_w = 1287,
  CODE_FOR_msa_maddv_h = 1288,
  CODE_FOR_msa_maddv_b = 1289,
  CODE_FOR_msa_msubv_d = 1290,
  CODE_FOR_msa_msubv_w = 1291,
  CODE_FOR_msa_msubv_h = 1292,
  CODE_FOR_msa_msubv_b = 1293,
  CODE_FOR_divv2di3 = 1294,
  CODE_FOR_divv4si3 = 1295,
  CODE_FOR_divv8hi3 = 1296,
  CODE_FOR_divv16qi3 = 1297,
  CODE_FOR_udivv2di3 = 1298,
  CODE_FOR_udivv4si3 = 1299,
  CODE_FOR_udivv8hi3 = 1300,
  CODE_FOR_udivv16qi3 = 1301,
  CODE_FOR_modv2di3 = 1302,
  CODE_FOR_modv4si3 = 1303,
  CODE_FOR_modv8hi3 = 1304,
  CODE_FOR_modv16qi3 = 1305,
  CODE_FOR_umodv2di3 = 1306,
  CODE_FOR_umodv4si3 = 1307,
  CODE_FOR_umodv8hi3 = 1308,
  CODE_FOR_umodv16qi3 = 1309,
  CODE_FOR_xorv2di3 = 1310,
  CODE_FOR_xorv4si3 = 1311,
  CODE_FOR_xorv8hi3 = 1312,
  CODE_FOR_xorv16qi3 = 1313,
  CODE_FOR_iorv2di3 = 1314,
  CODE_FOR_iorv4si3 = 1315,
  CODE_FOR_iorv8hi3 = 1316,
  CODE_FOR_iorv16qi3 = 1317,
  CODE_FOR_andv2di3 = 1318,
  CODE_FOR_andv4si3 = 1319,
  CODE_FOR_andv8hi3 = 1320,
  CODE_FOR_andv16qi3 = 1321,
  CODE_FOR_one_cmplv2di2 = 1322,
  CODE_FOR_one_cmplv4si2 = 1323,
  CODE_FOR_one_cmplv8hi2 = 1324,
  CODE_FOR_one_cmplv16qi2 = 1325,
  CODE_FOR_vlshrv2di3 = 1326,
  CODE_FOR_vlshrv4si3 = 1327,
  CODE_FOR_vlshrv8hi3 = 1328,
  CODE_FOR_vlshrv16qi3 = 1329,
  CODE_FOR_vashrv2di3 = 1330,
  CODE_FOR_vashrv4si3 = 1331,
  CODE_FOR_vashrv8hi3 = 1332,
  CODE_FOR_vashrv16qi3 = 1333,
  CODE_FOR_vashlv2di3 = 1334,
  CODE_FOR_vashlv4si3 = 1335,
  CODE_FOR_vashlv8hi3 = 1336,
  CODE_FOR_vashlv16qi3 = 1337,
  CODE_FOR_addv2df3 = 1338,
  CODE_FOR_addv4sf3 = 1339,
  CODE_FOR_subv2df3 = 1340,
  CODE_FOR_subv4sf3 = 1341,
  CODE_FOR_mulv2df3 = 1342,
  CODE_FOR_mulv4sf3 = 1343,
  CODE_FOR_divv2df3 = 1344,
  CODE_FOR_divv4sf3 = 1345,
  CODE_FOR_fmav2df4 = 1346,
  CODE_FOR_fmav4sf4 = 1347,
  CODE_FOR_fnmav2df4 = 1348,
  CODE_FOR_fnmav4sf4 = 1349,
  CODE_FOR_sqrtv2df2 = 1350,
  CODE_FOR_sqrtv4sf2 = 1351,
  CODE_FOR_msa_add_a_d = 1352,
  CODE_FOR_msa_add_a_w = 1353,
  CODE_FOR_msa_add_a_h = 1354,
  CODE_FOR_msa_add_a_b = 1355,
  CODE_FOR_msa_adds_a_d = 1356,
  CODE_FOR_msa_adds_a_w = 1357,
  CODE_FOR_msa_adds_a_h = 1358,
  CODE_FOR_msa_adds_a_b = 1359,
  CODE_FOR_ssaddv2di3 = 1360,
  CODE_FOR_ssaddv4si3 = 1361,
  CODE_FOR_ssaddv8hi3 = 1362,
  CODE_FOR_ssaddv16qi3 = 1363,
  CODE_FOR_usaddv2di3 = 1364,
  CODE_FOR_usaddv4si3 = 1365,
  CODE_FOR_usaddv8hi3 = 1366,
  CODE_FOR_usaddv16qi3 = 1367,
  CODE_FOR_msa_asub_s_d = 1368,
  CODE_FOR_msa_asub_s_w = 1369,
  CODE_FOR_msa_asub_s_h = 1370,
  CODE_FOR_msa_asub_s_b = 1371,
  CODE_FOR_msa_asub_u_d = 1372,
  CODE_FOR_msa_asub_u_w = 1373,
  CODE_FOR_msa_asub_u_h = 1374,
  CODE_FOR_msa_asub_u_b = 1375,
  CODE_FOR_msa_ave_s_d = 1376,
  CODE_FOR_msa_ave_s_w = 1377,
  CODE_FOR_msa_ave_s_h = 1378,
  CODE_FOR_msa_ave_s_b = 1379,
  CODE_FOR_msa_ave_u_d = 1380,
  CODE_FOR_msa_ave_u_w = 1381,
  CODE_FOR_msa_ave_u_h = 1382,
  CODE_FOR_msa_ave_u_b = 1383,
  CODE_FOR_msa_aver_s_d = 1384,
  CODE_FOR_msa_aver_s_w = 1385,
  CODE_FOR_msa_aver_s_h = 1386,
  CODE_FOR_msa_aver_s_b = 1387,
  CODE_FOR_msa_aver_u_d = 1388,
  CODE_FOR_msa_aver_u_w = 1389,
  CODE_FOR_msa_aver_u_h = 1390,
  CODE_FOR_msa_aver_u_b = 1391,
  CODE_FOR_msa_bclr_d = 1392,
  CODE_FOR_msa_bclr_w = 1393,
  CODE_FOR_msa_bclr_h = 1394,
  CODE_FOR_msa_bclr_b = 1395,
  CODE_FOR_msa_bclri_d = 1396,
  CODE_FOR_msa_bclri_w = 1397,
  CODE_FOR_msa_bclri_h = 1398,
  CODE_FOR_msa_bclri_b = 1399,
  CODE_FOR_msa_binsl_d = 1400,
  CODE_FOR_msa_binsl_w = 1401,
  CODE_FOR_msa_binsl_h = 1402,
  CODE_FOR_msa_binsl_b = 1403,
  CODE_FOR_msa_binsli_d = 1404,
  CODE_FOR_msa_binsli_w = 1405,
  CODE_FOR_msa_binsli_h = 1406,
  CODE_FOR_msa_binsli_b = 1407,
  CODE_FOR_msa_binsr_d = 1408,
  CODE_FOR_msa_binsr_w = 1409,
  CODE_FOR_msa_binsr_h = 1410,
  CODE_FOR_msa_binsr_b = 1411,
  CODE_FOR_msa_binsri_d = 1412,
  CODE_FOR_msa_binsri_w = 1413,
  CODE_FOR_msa_binsri_h = 1414,
  CODE_FOR_msa_binsri_b = 1415,
  CODE_FOR_msa_bmnz_d = 1416,
  CODE_FOR_msa_bmnz_w = 1417,
  CODE_FOR_msa_bmnz_h = 1418,
  CODE_FOR_msa_bmnz_b = 1419,
  CODE_FOR_msa_bmz_d = 1420,
  CODE_FOR_msa_bmz_w = 1421,
  CODE_FOR_msa_bmz_h = 1422,
  CODE_FOR_msa_bmz_b = 1423,
  CODE_FOR_msa_bneg_d = 1424,
  CODE_FOR_msa_bneg_w = 1425,
  CODE_FOR_msa_bneg_h = 1426,
  CODE_FOR_msa_bneg_b = 1427,
  CODE_FOR_msa_bnegi_d = 1428,
  CODE_FOR_msa_bnegi_w = 1429,
  CODE_FOR_msa_bnegi_h = 1430,
  CODE_FOR_msa_bnegi_b = 1431,
  CODE_FOR_msa_bsel_d = 1432,
  CODE_FOR_msa_bsel_w = 1433,
  CODE_FOR_msa_bsel_h = 1434,
  CODE_FOR_msa_bsel_b = 1435,
  CODE_FOR_msa_bset_d = 1436,
  CODE_FOR_msa_bset_w = 1437,
  CODE_FOR_msa_bset_h = 1438,
  CODE_FOR_msa_bset_b = 1439,
  CODE_FOR_msa_bseti_d = 1440,
  CODE_FOR_msa_bseti_w = 1441,
  CODE_FOR_msa_bseti_h = 1442,
  CODE_FOR_msa_bseti_b = 1443,
  CODE_FOR_msa_ceq_d = 1444,
  CODE_FOR_msa_cle_s_d = 1445,
  CODE_FOR_msa_cle_u_d = 1446,
  CODE_FOR_msa_clt_s_d = 1447,
  CODE_FOR_msa_clt_u_d = 1448,
  CODE_FOR_msa_ceq_w = 1449,
  CODE_FOR_msa_cle_s_w = 1450,
  CODE_FOR_msa_cle_u_w = 1451,
  CODE_FOR_msa_clt_s_w = 1452,
  CODE_FOR_msa_clt_u_w = 1453,
  CODE_FOR_msa_ceq_h = 1454,
  CODE_FOR_msa_cle_s_h = 1455,
  CODE_FOR_msa_cle_u_h = 1456,
  CODE_FOR_msa_clt_s_h = 1457,
  CODE_FOR_msa_clt_u_h = 1458,
  CODE_FOR_msa_ceq_b = 1459,
  CODE_FOR_msa_cle_s_b = 1460,
  CODE_FOR_msa_cle_u_b = 1461,
  CODE_FOR_msa_clt_s_b = 1462,
  CODE_FOR_msa_clt_u_b = 1463,
  CODE_FOR_msa_dotp_s_d = 1464,
  CODE_FOR_msa_dotp_u_d = 1465,
  CODE_FOR_msa_dotp_s_w = 1466,
  CODE_FOR_msa_dotp_u_w = 1467,
  CODE_FOR_msa_dotp_s_h = 1468,
  CODE_FOR_msa_dotp_u_h = 1469,
  CODE_FOR_msa_dpadd_s_d = 1470,
  CODE_FOR_msa_dpadd_u_d = 1471,
  CODE_FOR_msa_dpadd_s_w = 1472,
  CODE_FOR_msa_dpadd_u_w = 1473,
  CODE_FOR_msa_dpadd_s_h = 1474,
  CODE_FOR_msa_dpadd_u_h = 1475,
  CODE_FOR_msa_dpsub_s_d = 1476,
  CODE_FOR_msa_dpsub_u_d = 1477,
  CODE_FOR_msa_dpsub_s_w = 1478,
  CODE_FOR_msa_dpsub_u_w = 1479,
  CODE_FOR_msa_dpsub_s_h = 1480,
  CODE_FOR_msa_dpsub_u_h = 1481,
  CODE_FOR_msa_fclass_d = 1482,
  CODE_FOR_msa_fclass_w = 1483,
  CODE_FOR_msa_fcaf_d = 1484,
  CODE_FOR_msa_fcaf_w = 1485,
  CODE_FOR_msa_fcune_d = 1486,
  CODE_FOR_msa_fcune_w = 1487,
  CODE_FOR_msa_fcun_d = 1488,
  CODE_FOR_msa_fcor_d = 1489,
  CODE_FOR_msa_fceq_d = 1490,
  CODE_FOR_msa_fcne_d = 1491,
  CODE_FOR_msa_fcle_d = 1492,
  CODE_FOR_msa_fclt_d = 1493,
  CODE_FOR_msa_fcueq_d = 1494,
  CODE_FOR_msa_fcule_d = 1495,
  CODE_FOR_msa_fcult_d = 1496,
  CODE_FOR_msa_fcun_w = 1497,
  CODE_FOR_msa_fcor_w = 1498,
  CODE_FOR_msa_fceq_w = 1499,
  CODE_FOR_msa_fcne_w = 1500,
  CODE_FOR_msa_fcle_w = 1501,
  CODE_FOR_msa_fclt_w = 1502,
  CODE_FOR_msa_fcueq_w = 1503,
  CODE_FOR_msa_fcule_w = 1504,
  CODE_FOR_msa_fcult_w = 1505,
  CODE_FOR_msa_fsaf_d = 1506,
  CODE_FOR_msa_fsun_d = 1507,
  CODE_FOR_msa_fsor_d = 1508,
  CODE_FOR_msa_fseq_d = 1509,
  CODE_FOR_msa_fsne_d = 1510,
  CODE_FOR_msa_fsueq_d = 1511,
  CODE_FOR_msa_fsune_d = 1512,
  CODE_FOR_msa_fsule_d = 1513,
  CODE_FOR_msa_fsult_d = 1514,
  CODE_FOR_msa_fsle_d = 1515,
  CODE_FOR_msa_fslt_d = 1516,
  CODE_FOR_msa_fsaf_w = 1517,
  CODE_FOR_msa_fsun_w = 1518,
  CODE_FOR_msa_fsor_w = 1519,
  CODE_FOR_msa_fseq_w = 1520,
  CODE_FOR_msa_fsne_w = 1521,
  CODE_FOR_msa_fsueq_w = 1522,
  CODE_FOR_msa_fsune_w = 1523,
  CODE_FOR_msa_fsule_w = 1524,
  CODE_FOR_msa_fsult_w = 1525,
  CODE_FOR_msa_fsle_w = 1526,
  CODE_FOR_msa_fslt_w = 1527,
  CODE_FOR_msa_fexp2_d = 1528,
  CODE_FOR_msa_fexp2_w = 1529,
  CODE_FOR_floatv2div2df2 = 1530,
  CODE_FOR_floatv4siv4sf2 = 1531,
  CODE_FOR_floatunsv2div2df2 = 1532,
  CODE_FOR_floatunsv4siv4sf2 = 1533,
  CODE_FOR_msa_ffql_d = 1534,
  CODE_FOR_msa_ffql_w = 1535,
  CODE_FOR_msa_ffqr_d = 1536,
  CODE_FOR_msa_ffqr_w = 1537,
  CODE_FOR_msa_fill_d_f = 1538,
  CODE_FOR_msa_fill_w_f = 1539,
  CODE_FOR_msa_fill_d = 1540,
  CODE_FOR_msa_fill_w = 1541,
  CODE_FOR_msa_fill_h = 1542,
  CODE_FOR_msa_fill_b = 1543,
  CODE_FOR_msa_flog2_d = 1544,
  CODE_FOR_msa_flog2_w = 1545,
  CODE_FOR_smaxv2df3 = 1546,
  CODE_FOR_smaxv4sf3 = 1547,
  CODE_FOR_msa_fmax_a_d = 1548,
  CODE_FOR_msa_fmax_a_w = 1549,
  CODE_FOR_sminv2df3 = 1550,
  CODE_FOR_sminv4sf3 = 1551,
  CODE_FOR_msa_fmin_a_d = 1552,
  CODE_FOR_msa_fmin_a_w = 1553,
  CODE_FOR_msa_frcp_d = 1554,
  CODE_FOR_msa_frcp_w = 1555,
  CODE_FOR_msa_frint_d = 1556,
  CODE_FOR_msa_frint_w = 1557,
  CODE_FOR_msa_frsqrt_d = 1558,
  CODE_FOR_msa_frsqrt_w = 1559,
  CODE_FOR_msa_ftint_s_d = 1560,
  CODE_FOR_msa_ftint_s_w = 1561,
  CODE_FOR_msa_ftint_u_d = 1562,
  CODE_FOR_msa_ftint_u_w = 1563,
  CODE_FOR_fix_truncv2dfv2di2 = 1564,
  CODE_FOR_fix_truncv4sfv4si2 = 1565,
  CODE_FOR_fixuns_truncv2dfv2di2 = 1566,
  CODE_FOR_fixuns_truncv4sfv4si2 = 1567,
  CODE_FOR_msa_ftq_h = 1568,
  CODE_FOR_msa_ftq_w = 1569,
  CODE_FOR_msa_hadd_s_h = 1570,
  CODE_FOR_msa_hadd_u_h = 1571,
  CODE_FOR_msa_hsub_s_h = 1572,
  CODE_FOR_msa_hsub_u_h = 1573,
  CODE_FOR_msa_hadd_s_w = 1574,
  CODE_FOR_msa_hadd_u_w = 1575,
  CODE_FOR_msa_hsub_s_w = 1576,
  CODE_FOR_msa_hsub_u_w = 1577,
  CODE_FOR_msa_hadd_s_d = 1578,
  CODE_FOR_msa_hadd_u_d = 1579,
  CODE_FOR_msa_hsub_s_d = 1580,
  CODE_FOR_msa_hsub_u_d = 1581,
  CODE_FOR_msa_ilvev_b = 1582,
  CODE_FOR_msa_ilvev_h = 1583,
  CODE_FOR_msa_ilvev_w = 1584,
  CODE_FOR_msa_ilvev_w_f = 1585,
  CODE_FOR_msa_ilvl_b = 1586,
  CODE_FOR_msa_ilvl_h = 1587,
  CODE_FOR_msa_ilvl_w = 1588,
  CODE_FOR_msa_ilvl_w_f = 1589,
  CODE_FOR_msa_ilvl_d = 1590,
  CODE_FOR_msa_ilvl_d_f = 1591,
  CODE_FOR_msa_ilvod_b = 1592,
  CODE_FOR_msa_ilvod_h = 1593,
  CODE_FOR_msa_ilvod_w = 1594,
  CODE_FOR_msa_ilvod_w_f = 1595,
  CODE_FOR_msa_ilvr_b = 1596,
  CODE_FOR_msa_ilvr_h = 1597,
  CODE_FOR_msa_ilvr_w = 1598,
  CODE_FOR_msa_ilvr_w_f = 1599,
  CODE_FOR_msa_ilvr_d = 1600,
  CODE_FOR_msa_ilvr_d_f = 1601,
  CODE_FOR_msa_madd_q_w = 1602,
  CODE_FOR_msa_madd_q_h = 1603,
  CODE_FOR_msa_maddr_q_w = 1604,
  CODE_FOR_msa_maddr_q_h = 1605,
  CODE_FOR_msa_max_a_d = 1606,
  CODE_FOR_msa_max_a_w = 1607,
  CODE_FOR_msa_max_a_h = 1608,
  CODE_FOR_msa_max_a_b = 1609,
  CODE_FOR_smaxv2di3 = 1610,
  CODE_FOR_smaxv4si3 = 1611,
  CODE_FOR_smaxv8hi3 = 1612,
  CODE_FOR_smaxv16qi3 = 1613,
  CODE_FOR_umaxv2di3 = 1614,
  CODE_FOR_umaxv4si3 = 1615,
  CODE_FOR_umaxv8hi3 = 1616,
  CODE_FOR_umaxv16qi3 = 1617,
  CODE_FOR_msa_min_a_d = 1618,
  CODE_FOR_msa_min_a_w = 1619,
  CODE_FOR_msa_min_a_h = 1620,
  CODE_FOR_msa_min_a_b = 1621,
  CODE_FOR_sminv2di3 = 1622,
  CODE_FOR_sminv4si3 = 1623,
  CODE_FOR_sminv8hi3 = 1624,
  CODE_FOR_sminv16qi3 = 1625,
  CODE_FOR_uminv2di3 = 1626,
  CODE_FOR_uminv4si3 = 1627,
  CODE_FOR_uminv8hi3 = 1628,
  CODE_FOR_uminv16qi3 = 1629,
  CODE_FOR_msa_msub_q_w = 1630,
  CODE_FOR_msa_msub_q_h = 1631,
  CODE_FOR_msa_msubr_q_w = 1632,
  CODE_FOR_msa_msubr_q_h = 1633,
  CODE_FOR_msa_mul_q_w = 1634,
  CODE_FOR_msa_mul_q_h = 1635,
  CODE_FOR_msa_mulr_q_w = 1636,
  CODE_FOR_msa_mulr_q_h = 1637,
  CODE_FOR_msa_nloc_d = 1638,
  CODE_FOR_msa_nloc_w = 1639,
  CODE_FOR_msa_nloc_h = 1640,
  CODE_FOR_msa_nloc_b = 1641,
  CODE_FOR_clzv2di2 = 1642,
  CODE_FOR_clzv4si2 = 1643,
  CODE_FOR_clzv8hi2 = 1644,
  CODE_FOR_clzv16qi2 = 1645,
  CODE_FOR_msa_nor_d = 1646,
  CODE_FOR_msa_nor_w = 1647,
  CODE_FOR_msa_nor_h = 1648,
  CODE_FOR_msa_nor_b = 1649,
  CODE_FOR_msa_pckev_b = 1650,
  CODE_FOR_msa_pckev_h = 1651,
  CODE_FOR_msa_pckev_w = 1652,
  CODE_FOR_msa_pckev_w_f = 1653,
  CODE_FOR_msa_pckod_b = 1654,
  CODE_FOR_msa_pckod_h = 1655,
  CODE_FOR_msa_pckod_w = 1656,
  CODE_FOR_msa_pckod_w_f = 1657,
  CODE_FOR_popcountv2di2 = 1658,
  CODE_FOR_popcountv4si2 = 1659,
  CODE_FOR_popcountv8hi2 = 1660,
  CODE_FOR_popcountv16qi2 = 1661,
  CODE_FOR_msa_sat_s_d = 1662,
  CODE_FOR_msa_sat_s_w = 1663,
  CODE_FOR_msa_sat_s_h = 1664,
  CODE_FOR_msa_sat_s_b = 1665,
  CODE_FOR_msa_sat_u_d = 1666,
  CODE_FOR_msa_sat_u_w = 1667,
  CODE_FOR_msa_sat_u_h = 1668,
  CODE_FOR_msa_sat_u_b = 1669,
  CODE_FOR_msa_shf_d_f = 1670,
  CODE_FOR_msa_shf_w_f = 1671,
  CODE_FOR_msa_shf_d = 1672,
  CODE_FOR_msa_shf_w = 1673,
  CODE_FOR_msa_shf_h = 1674,
  CODE_FOR_msa_shf_b = 1675,
  CODE_FOR_msa_srar_d = 1676,
  CODE_FOR_msa_srar_w = 1677,
  CODE_FOR_msa_srar_h = 1678,
  CODE_FOR_msa_srar_b = 1679,
  CODE_FOR_msa_srari_d = 1680,
  CODE_FOR_msa_srari_w = 1681,
  CODE_FOR_msa_srari_h = 1682,
  CODE_FOR_msa_srari_b = 1683,
  CODE_FOR_msa_srlr_d = 1684,
  CODE_FOR_msa_srlr_w = 1685,
  CODE_FOR_msa_srlr_h = 1686,
  CODE_FOR_msa_srlr_b = 1687,
  CODE_FOR_msa_srlri_d = 1688,
  CODE_FOR_msa_srlri_w = 1689,
  CODE_FOR_msa_srlri_h = 1690,
  CODE_FOR_msa_srlri_b = 1691,
  CODE_FOR_msa_subs_s_d = 1692,
  CODE_FOR_msa_subs_s_w = 1693,
  CODE_FOR_msa_subs_s_h = 1694,
  CODE_FOR_msa_subs_s_b = 1695,
  CODE_FOR_msa_subs_u_d = 1696,
  CODE_FOR_msa_subs_u_w = 1697,
  CODE_FOR_msa_subs_u_h = 1698,
  CODE_FOR_msa_subs_u_b = 1699,
  CODE_FOR_msa_subsuu_s_d = 1700,
  CODE_FOR_msa_subsuu_s_w = 1701,
  CODE_FOR_msa_subsuu_s_h = 1702,
  CODE_FOR_msa_subsuu_s_b = 1703,
  CODE_FOR_msa_subsus_u_d = 1704,
  CODE_FOR_msa_subsus_u_w = 1705,
  CODE_FOR_msa_subsus_u_h = 1706,
  CODE_FOR_msa_subsus_u_b = 1707,
  CODE_FOR_msa_sld_d_f = 1708,
  CODE_FOR_msa_sld_w_f = 1709,
  CODE_FOR_msa_sld_d = 1710,
  CODE_FOR_msa_sld_w = 1711,
  CODE_FOR_msa_sld_h = 1712,
  CODE_FOR_msa_sld_b = 1713,
  CODE_FOR_msa_sldi_d_f = 1714,
  CODE_FOR_msa_sldi_w_f = 1715,
  CODE_FOR_msa_sldi_d = 1716,
  CODE_FOR_msa_sldi_w = 1717,
  CODE_FOR_msa_sldi_h = 1718,
  CODE_FOR_msa_sldi_b = 1719,
  CODE_FOR_msa_splat_d_f = 1720,
  CODE_FOR_msa_splat_w_f = 1721,
  CODE_FOR_msa_splat_d = 1722,
  CODE_FOR_msa_splat_w = 1723,
  CODE_FOR_msa_splat_h = 1724,
  CODE_FOR_msa_splat_b = 1725,
  CODE_FOR_msa_splati_d_f = 1726,
  CODE_FOR_msa_splati_w_f = 1727,
  CODE_FOR_msa_splati_d = 1728,
  CODE_FOR_msa_splati_w = 1729,
  CODE_FOR_msa_splati_h = 1730,
  CODE_FOR_msa_splati_b = 1731,
  CODE_FOR_msa_splati_d_f_scalar = 1732,
  CODE_FOR_msa_splati_w_f_scalar = 1733,
  CODE_FOR_msa_cfcmsa = 1734,
  CODE_FOR_msa_ctcmsa = 1735,
  CODE_FOR_msa_fexdo_h = 1736,
  CODE_FOR_vec_pack_trunc_v2df = 1737,
  CODE_FOR_msa_fexupl_w = 1738,
  CODE_FOR_msa_fexupl_d = 1739,
  CODE_FOR_msa_fexupr_w = 1740,
  CODE_FOR_msa_fexupr_d = 1741,
  CODE_FOR_msa_bz_d_f = 1742,
  CODE_FOR_msa_bnz_d_f = 1743,
  CODE_FOR_msa_bz_w_f = 1744,
  CODE_FOR_msa_bnz_w_f = 1745,
  CODE_FOR_msa_bz_d = 1746,
  CODE_FOR_msa_bnz_d = 1747,
  CODE_FOR_msa_bz_w = 1748,
  CODE_FOR_msa_bnz_w = 1749,
  CODE_FOR_msa_bz_h = 1750,
  CODE_FOR_msa_bnz_h = 1751,
  CODE_FOR_msa_bz_b = 1752,
  CODE_FOR_msa_bnz_b = 1753,
  CODE_FOR_msa_bz_v_d_f = 1754,
  CODE_FOR_msa_bnz_v_d_f = 1755,
  CODE_FOR_msa_bz_v_w_f = 1756,
  CODE_FOR_msa_bnz_v_w_f = 1757,
  CODE_FOR_msa_bz_v_d = 1758,
  CODE_FOR_msa_bnz_v_d = 1759,
  CODE_FOR_msa_bz_v_w = 1760,
  CODE_FOR_msa_bnz_v_w = 1761,
  CODE_FOR_msa_bz_v_h = 1762,
  CODE_FOR_msa_bnz_v_h = 1763,
  CODE_FOR_msa_bz_v_b = 1764,
  CODE_FOR_msa_bnz_v_b = 1765,
  CODE_FOR_ctrapsi4 = 1766,
  CODE_FOR_ctrapdi4 = 1767,
  CODE_FOR_addsi3 = 1768,
  CODE_FOR_adddi3 = 1769,
  CODE_FOR_mulsf3 = 1770,
  CODE_FOR_muldf3 = 1771,
  CODE_FOR_mulsi3 = 1772,
  CODE_FOR_muldi3 = 1773,
  CODE_FOR_mulsidi3 = 1774,
  CODE_FOR_umulsidi3 = 1775,
  CODE_FOR_mulsidi3_32bit_r6 = 1776,
  CODE_FOR_umulsidi3_32bit_r6 = 1777,
  CODE_FOR_mulsidi3_32bit_mips16 = 1778,
  CODE_FOR_umulsidi3_32bit_mips16 = 1779,
  CODE_FOR_mulsidi3_64bit_mips16 = 1780,
  CODE_FOR_umulsidi3_64bit_mips16 = 1781,
  CODE_FOR_mulsidi3_64bit_split = 1782,
  CODE_FOR_umulsidi3_64bit_split = 1783,
  CODE_FOR_smulsi3_highpart = 1784,
  CODE_FOR_umulsi3_highpart = 1785,
  CODE_FOR_smulsi3_highpart_split = 1786,
  CODE_FOR_umulsi3_highpart_split = 1787,
  CODE_FOR_smuldi3_highpart = 1788,
  CODE_FOR_umuldi3_highpart = 1789,
  CODE_FOR_smuldi3_highpart_split = 1790,
  CODE_FOR_umuldi3_highpart_split = 1791,
  CODE_FOR_mulditi3 = 1792,
  CODE_FOR_umulditi3 = 1793,
  CODE_FOR_fmasf4 = 1794,
  CODE_FOR_fmadf4 = 1795,
  CODE_FOR_fmav2sf4 = 1796,
  CODE_FOR_fmssf4 = 1797,
  CODE_FOR_fmsdf4 = 1798,
  CODE_FOR_fmsv2sf4 = 1799,
  CODE_FOR_fnmasf4 = 1800,
  CODE_FOR_fnmadf4 = 1801,
  CODE_FOR_fnmav2sf4 = 1802,
  CODE_FOR_fnmssf4 = 1803,
  CODE_FOR_fnmsdf4 = 1804,
  CODE_FOR_fnmsv2sf4 = 1805,
  CODE_FOR_divsf3 = 1806,
  CODE_FOR_divdf3 = 1807,
  CODE_FOR_divv2sf3 = 1808,
  CODE_FOR_divmodsi4 = 1809,
  CODE_FOR_divmoddi4 = 1810,
  CODE_FOR_udivmodsi4 = 1811,
  CODE_FOR_udivmoddi4 = 1812,
  CODE_FOR_divmodsi4_split = 1813,
  CODE_FOR_udivmodsi4_split = 1814,
  CODE_FOR_divmoddi4_split = 1815,
  CODE_FOR_udivmoddi4_split = 1816,
  CODE_FOR_andsi3 = 1817,
  CODE_FOR_anddi3 = 1818,
  CODE_FOR_iorsi3 = 1819,
  CODE_FOR_iordi3 = 1820,
  CODE_FOR_xorsi3 = 1821,
  CODE_FOR_xordi3 = 1822,
  CODE_FOR_zero_extendsidi2 = 1823,
  CODE_FOR_zero_extendqisi2 = 1824,
  CODE_FOR_zero_extendqidi2 = 1825,
  CODE_FOR_zero_extendhisi2 = 1826,
  CODE_FOR_zero_extendhidi2 = 1827,
  CODE_FOR_zero_extendqihi2 = 1828,
  CODE_FOR_extendqisi2 = 1829,
  CODE_FOR_extendqidi2 = 1830,
  CODE_FOR_extendhisi2 = 1831,
  CODE_FOR_extendhidi2 = 1832,
  CODE_FOR_extendqihi2 = 1833,
  CODE_FOR_fix_truncdfsi2 = 1834,
  CODE_FOR_fix_truncsfsi2 = 1835,
  CODE_FOR_fixuns_truncdfsi2 = 1836,
  CODE_FOR_fixuns_truncdfdi2 = 1837,
  CODE_FOR_fixuns_truncsfsi2 = 1838,
  CODE_FOR_fixuns_truncsfdi2 = 1839,
  CODE_FOR_extvmisalignsi = 1840,
  CODE_FOR_extvmisaligndi = 1841,
  CODE_FOR_extvsi = 1842,
  CODE_FOR_extvdi = 1843,
  CODE_FOR_extzvmisalignsi = 1844,
  CODE_FOR_extzvmisaligndi = 1845,
  CODE_FOR_extzvsi = 1846,
  CODE_FOR_extzvdi = 1847,
  CODE_FOR_insvmisalignsi = 1848,
  CODE_FOR_insvmisaligndi = 1849,
  CODE_FOR_insvsi = 1850,
  CODE_FOR_insvdi = 1851,
  CODE_FOR_movmisalignhi = 1852,
  CODE_FOR_movmisalignsi = 1853,
  CODE_FOR_movmisalignsf = 1854,
  CODE_FOR_movmisaligndf = 1855,
  CODE_FOR_unspec_got_si = 1856,
  CODE_FOR_unspec_got_di = 1857,
  CODE_FOR_movdi = 1858,
  CODE_FOR_movsi = 1859,
  CODE_FOR_movv2hi = 1860,
  CODE_FOR_movv4qi = 1861,
  CODE_FOR_movv2hq = 1862,
  CODE_FOR_movv2uhq = 1863,
  CODE_FOR_movv2ha = 1864,
  CODE_FOR_movv2uha = 1865,
  CODE_FOR_movv4qq = 1866,
  CODE_FOR_movv4uqq = 1867,
  CODE_FOR_movhi = 1868,
  CODE_FOR_movqi = 1869,
  CODE_FOR_movsf = 1870,
  CODE_FOR_movdf = 1871,
  CODE_FOR_movti = 1872,
  CODE_FOR_movtf = 1873,
  CODE_FOR_movv2sf = 1874,
  CODE_FOR_move_doubleword_fprdf = 1875,
  CODE_FOR_move_doubleword_fprdi = 1876,
  CODE_FOR_move_doubleword_fprv2sf = 1877,
  CODE_FOR_move_doubleword_fprv2si = 1878,
  CODE_FOR_move_doubleword_fprv4hi = 1879,
  CODE_FOR_move_doubleword_fprv8qi = 1880,
  CODE_FOR_move_doubleword_fprtf = 1881,
  CODE_FOR_load_const_gp_si = 1882,
  CODE_FOR_load_const_gp_di = 1883,
  CODE_FOR_clear_cache = 1884,
  CODE_FOR_cpymemsi = 1885,
  CODE_FOR_ashlsi3 = 1886,
  CODE_FOR_ashrsi3 = 1887,
  CODE_FOR_lshrsi3 = 1888,
  CODE_FOR_ashldi3 = 1889,
  CODE_FOR_ashrdi3 = 1890,
  CODE_FOR_lshrdi3 = 1891,
  CODE_FOR_cbranchsi4 = 1892,
  CODE_FOR_cbranchdi4 = 1893,
  CODE_FOR_cbranchsf4 = 1894,
  CODE_FOR_cbranchdf4 = 1895,
  CODE_FOR_condjump = 1896,
  CODE_FOR_cstoresi4 = 1897,
  CODE_FOR_cstoredi4 = 1898,
  CODE_FOR_jump = 1899,
  CODE_FOR_indirect_jump = 1900,
  CODE_FOR_tablejump = 1901,
  CODE_FOR_casesi = 1902,
  CODE_FOR_builtin_setjmp_setup = 1903,
  CODE_FOR_builtin_longjmp = 1904,
  CODE_FOR_prologue = 1905,
  CODE_FOR_epilogue = 1906,
  CODE_FOR_sibcall_epilogue = 1907,
  CODE_FOR_return = 1908,
  CODE_FOR_simple_return = 1909,
  CODE_FOR_eh_return = 1910,
  CODE_FOR_exception_receiver = 1911,
  CODE_FOR_nonlocal_goto_receiver = 1912,
  CODE_FOR_sibcall = 1913,
  CODE_FOR_sibcall_value = 1914,
  CODE_FOR_call = 1915,
  CODE_FOR_call_value = 1916,
  CODE_FOR_untyped_call = 1917,
  CODE_FOR_movsicc = 1918,
  CODE_FOR_movdicc = 1919,
  CODE_FOR_movsfcc = 1920,
  CODE_FOR_movdfcc = 1921,
  CODE_FOR_speculation_barrier = 1922,
  CODE_FOR_get_thread_pointersi = 1923,
  CODE_FOR_get_thread_pointerdi = 1924,
  CODE_FOR_mips_get_fcsr = 1925,
  CODE_FOR_mips_set_fcsr = 1926,
  CODE_FOR_memory_barrier = 1927,
  CODE_FOR_sync_compare_and_swapqi = 1928,
  CODE_FOR_sync_compare_and_swaphi = 1929,
  CODE_FOR_sync_addqi = 1930,
  CODE_FOR_sync_subqi = 1931,
  CODE_FOR_sync_iorqi = 1932,
  CODE_FOR_sync_xorqi = 1933,
  CODE_FOR_sync_andqi = 1934,
  CODE_FOR_sync_addhi = 1935,
  CODE_FOR_sync_subhi = 1936,
  CODE_FOR_sync_iorhi = 1937,
  CODE_FOR_sync_xorhi = 1938,
  CODE_FOR_sync_andhi = 1939,
  CODE_FOR_sync_old_addqi = 1940,
  CODE_FOR_sync_old_subqi = 1941,
  CODE_FOR_sync_old_iorqi = 1942,
  CODE_FOR_sync_old_xorqi = 1943,
  CODE_FOR_sync_old_andqi = 1944,
  CODE_FOR_sync_old_addhi = 1945,
  CODE_FOR_sync_old_subhi = 1946,
  CODE_FOR_sync_old_iorhi = 1947,
  CODE_FOR_sync_old_xorhi = 1948,
  CODE_FOR_sync_old_andhi = 1949,
  CODE_FOR_sync_new_addqi = 1950,
  CODE_FOR_sync_new_subqi = 1951,
  CODE_FOR_sync_new_iorqi = 1952,
  CODE_FOR_sync_new_xorqi = 1953,
  CODE_FOR_sync_new_andqi = 1954,
  CODE_FOR_sync_new_addhi = 1955,
  CODE_FOR_sync_new_subhi = 1956,
  CODE_FOR_sync_new_iorhi = 1957,
  CODE_FOR_sync_new_xorhi = 1958,
  CODE_FOR_sync_new_andhi = 1959,
  CODE_FOR_sync_nandqi = 1960,
  CODE_FOR_sync_nandhi = 1961,
  CODE_FOR_sync_old_nandqi = 1962,
  CODE_FOR_sync_old_nandhi = 1963,
  CODE_FOR_sync_new_nandqi = 1964,
  CODE_FOR_sync_new_nandhi = 1965,
  CODE_FOR_sync_lock_test_and_setqi = 1966,
  CODE_FOR_sync_lock_test_and_sethi = 1967,
  CODE_FOR_atomic_exchangesi = 1968,
  CODE_FOR_atomic_exchangedi = 1969,
  CODE_FOR_atomic_fetch_addsi = 1970,
  CODE_FOR_atomic_fetch_adddi = 1971,
  CODE_FOR_movv2sfcc = 1972,
  CODE_FOR_mips_puu_ps = 1973,
  CODE_FOR_mips_pul_ps = 1974,
  CODE_FOR_mips_plu_ps = 1975,
  CODE_FOR_mips_pll_ps = 1976,
  CODE_FOR_vec_initv2sfsf = 1977,
  CODE_FOR_vec_setv2sf = 1978,
  CODE_FOR_mips_cvt_ps_s = 1979,
  CODE_FOR_mips_cvt_s_pl = 1980,
  CODE_FOR_mips_cvt_s_pu = 1981,
  CODE_FOR_reduc_plus_scal_v2sf = 1982,
  CODE_FOR_mips_abs_ps = 1983,
  CODE_FOR_scc_ps = 1984,
  CODE_FOR_single_cc = 1985,
  CODE_FOR_vcondv2sfv2sf = 1986,
  CODE_FOR_sminv2sf3 = 1987,
  CODE_FOR_smaxv2sf3 = 1988,
  CODE_FOR_reduc_smin_scal_v2sf = 1989,
  CODE_FOR_reduc_smax_scal_v2sf = 1990,
  CODE_FOR_mips_lbux = 1991,
  CODE_FOR_mips_lhx = 1992,
  CODE_FOR_mips_lwx = 1993,
  CODE_FOR_mips_ldx = 1994,
  CODE_FOR_mips_madd = 1995,
  CODE_FOR_mips_maddu = 1996,
  CODE_FOR_mips_msub = 1997,
  CODE_FOR_mips_msubu = 1998,
  CODE_FOR_movv2si = 1999,
  CODE_FOR_movv4hi = 2000,
  CODE_FOR_movv8qi = 2001,
  CODE_FOR_vec_initv2sisi = 2002,
  CODE_FOR_vec_initv4hihi = 2003,
  CODE_FOR_vec_initv8qiqi = 2004,
  CODE_FOR_vec_setv4hi = 2005,
  CODE_FOR_sdot_prodv2siv4hi = 2006,
  CODE_FOR_smaxv2si3 = 2007,
  CODE_FOR_smaxv8qi3 = 2008,
  CODE_FOR_sminv2si3 = 2009,
  CODE_FOR_sminv8qi3 = 2010,
  CODE_FOR_vec_unpacks_lo_v4hi = 2011,
  CODE_FOR_vec_unpacks_lo_v8qi = 2012,
  CODE_FOR_vec_unpacks_hi_v4hi = 2013,
  CODE_FOR_vec_unpacks_hi_v8qi = 2014,
  CODE_FOR_vec_unpacku_lo_v4hi = 2015,
  CODE_FOR_vec_unpacku_lo_v8qi = 2016,
  CODE_FOR_vec_unpacku_hi_v4hi = 2017,
  CODE_FOR_vec_unpacku_hi_v8qi = 2018,
  CODE_FOR_reduc_plus_scal_v2si = 2019,
  CODE_FOR_reduc_plus_scal_v4hi = 2020,
  CODE_FOR_reduc_plus_scal_v8qi = 2021,
  CODE_FOR_reduc_smax_scal_v2si = 2022,
  CODE_FOR_reduc_smax_scal_v4hi = 2023,
  CODE_FOR_reduc_smax_scal_v8qi = 2024,
  CODE_FOR_reduc_smin_scal_v2si = 2025,
  CODE_FOR_reduc_smin_scal_v4hi = 2026,
  CODE_FOR_reduc_smin_scal_v8qi = 2027,
  CODE_FOR_reduc_umax_scal_v8qi = 2028,
  CODE_FOR_reduc_umin_scal_v8qi = 2029,
  CODE_FOR_vec_initv2dfdf = 2030,
  CODE_FOR_vec_initv4sfsf = 2031,
  CODE_FOR_vec_initv2didi = 2032,
  CODE_FOR_vec_initv4sisi = 2033,
  CODE_FOR_vec_initv8hihi = 2034,
  CODE_FOR_vec_initv16qiqi = 2035,
  CODE_FOR_vec_unpacks_hi_v4sf = 2036,
  CODE_FOR_vec_unpacks_lo_v4sf = 2037,
  CODE_FOR_vec_unpacks_hi_v4si = 2038,
  CODE_FOR_vec_unpacks_hi_v8hi = 2039,
  CODE_FOR_vec_unpacks_hi_v16qi = 2040,
  CODE_FOR_vec_unpacks_lo_v4si = 2041,
  CODE_FOR_vec_unpacks_lo_v8hi = 2042,
  CODE_FOR_vec_unpacks_lo_v16qi = 2043,
  CODE_FOR_vec_unpacku_hi_v4si = 2044,
  CODE_FOR_vec_unpacku_hi_v8hi = 2045,
  CODE_FOR_vec_unpacku_hi_v16qi = 2046,
  CODE_FOR_vec_unpacku_lo_v4si = 2047,
  CODE_FOR_vec_unpacku_lo_v8hi = 2048,
  CODE_FOR_vec_unpacku_lo_v16qi = 2049,
  CODE_FOR_vec_extractv2didi = 2050,
  CODE_FOR_vec_extractv4sisi = 2051,
  CODE_FOR_vec_extractv8hihi = 2052,
  CODE_FOR_vec_extractv16qiqi = 2053,
  CODE_FOR_vec_extractv2dfdf = 2054,
  CODE_FOR_vec_extractv4sfsf = 2055,
  CODE_FOR_vec_setv2di = 2056,
  CODE_FOR_vec_setv4si = 2057,
  CODE_FOR_vec_setv8hi = 2058,
  CODE_FOR_vec_setv16qi = 2059,
  CODE_FOR_vec_setv2df = 2060,
  CODE_FOR_vec_setv4sf = 2061,
  CODE_FOR_vcond_mask_v2dfv2di = 2062,
  CODE_FOR_vcond_mask_v2dfv4si = 2063,
  CODE_FOR_vcond_mask_v2dfv8hi = 2064,
  CODE_FOR_vcond_mask_v2dfv16qi = 2065,
  CODE_FOR_vcond_mask_v4sfv2di = 2066,
  CODE_FOR_vcond_mask_v4sfv4si = 2067,
  CODE_FOR_vcond_mask_v4sfv8hi = 2068,
  CODE_FOR_vcond_mask_v4sfv16qi = 2069,
  CODE_FOR_vcond_mask_v2div2di = 2070,
  CODE_FOR_vcond_mask_v2div4si = 2071,
  CODE_FOR_vcond_mask_v2div8hi = 2072,
  CODE_FOR_vcond_mask_v2div16qi = 2073,
  CODE_FOR_vcond_mask_v4siv2di = 2074,
  CODE_FOR_vcond_mask_v4siv4si = 2075,
  CODE_FOR_vcond_mask_v4siv8hi = 2076,
  CODE_FOR_vcond_mask_v4siv16qi = 2077,
  CODE_FOR_vcond_mask_v8hiv2di = 2078,
  CODE_FOR_vcond_mask_v8hiv4si = 2079,
  CODE_FOR_vcond_mask_v8hiv8hi = 2080,
  CODE_FOR_vcond_mask_v8hiv16qi = 2081,
  CODE_FOR_vcond_mask_v16qiv2di = 2082,
  CODE_FOR_vcond_mask_v16qiv4si = 2083,
  CODE_FOR_vcond_mask_v16qiv8hi = 2084,
  CODE_FOR_vcond_mask_v16qiv16qi = 2085,
  CODE_FOR_vconduv2dfv2di = 2086,
  CODE_FOR_vconduv2dfv4si = 2087,
  CODE_FOR_vconduv2dfv8hi = 2088,
  CODE_FOR_vconduv2dfv16qi = 2089,
  CODE_FOR_vconduv4sfv2di = 2090,
  CODE_FOR_vconduv4sfv4si = 2091,
  CODE_FOR_vconduv4sfv8hi = 2092,
  CODE_FOR_vconduv4sfv16qi = 2093,
  CODE_FOR_vconduv2div2di = 2094,
  CODE_FOR_vconduv2div4si = 2095,
  CODE_FOR_vconduv2div8hi = 2096,
  CODE_FOR_vconduv2div16qi = 2097,
  CODE_FOR_vconduv4siv2di = 2098,
  CODE_FOR_vconduv4siv4si = 2099,
  CODE_FOR_vconduv4siv8hi = 2100,
  CODE_FOR_vconduv4siv16qi = 2101,
  CODE_FOR_vconduv8hiv2di = 2102,
  CODE_FOR_vconduv8hiv4si = 2103,
  CODE_FOR_vconduv8hiv8hi = 2104,
  CODE_FOR_vconduv8hiv16qi = 2105,
  CODE_FOR_vconduv16qiv2di = 2106,
  CODE_FOR_vconduv16qiv4si = 2107,
  CODE_FOR_vconduv16qiv8hi = 2108,
  CODE_FOR_vconduv16qiv16qi = 2109,
  CODE_FOR_vcondv2dfv2df = 2110,
  CODE_FOR_vcondv4sfv2df = 2111,
  CODE_FOR_vcondv2div2df = 2112,
  CODE_FOR_vcondv4siv2df = 2113,
  CODE_FOR_vcondv8hiv2df = 2114,
  CODE_FOR_vcondv16qiv2df = 2115,
  CODE_FOR_vcondv2dfv4sf = 2116,
  CODE_FOR_vcondv4sfv4sf = 2117,
  CODE_FOR_vcondv2div4sf = 2118,
  CODE_FOR_vcondv4siv4sf = 2119,
  CODE_FOR_vcondv8hiv4sf = 2120,
  CODE_FOR_vcondv16qiv4sf = 2121,
  CODE_FOR_vcondv2dfv2di = 2122,
  CODE_FOR_vcondv4sfv2di = 2123,
  CODE_FOR_vcondv2div2di = 2124,
  CODE_FOR_vcondv4siv2di = 2125,
  CODE_FOR_vcondv8hiv2di = 2126,
  CODE_FOR_vcondv16qiv2di = 2127,
  CODE_FOR_vcondv2dfv4si = 2128,
  CODE_FOR_vcondv4sfv4si = 2129,
  CODE_FOR_vcondv2div4si = 2130,
  CODE_FOR_vcondv4siv4si = 2131,
  CODE_FOR_vcondv8hiv4si = 2132,
  CODE_FOR_vcondv16qiv4si = 2133,
  CODE_FOR_vcondv2dfv8hi = 2134,
  CODE_FOR_vcondv4sfv8hi = 2135,
  CODE_FOR_vcondv2div8hi = 2136,
  CODE_FOR_vcondv4siv8hi = 2137,
  CODE_FOR_vcondv8hiv8hi = 2138,
  CODE_FOR_vcondv16qiv8hi = 2139,
  CODE_FOR_vcondv2dfv16qi = 2140,
  CODE_FOR_vcondv4sfv16qi = 2141,
  CODE_FOR_vcondv2div16qi = 2142,
  CODE_FOR_vcondv4siv16qi = 2143,
  CODE_FOR_vcondv8hiv16qi = 2144,
  CODE_FOR_vcondv16qiv16qi = 2145,
  CODE_FOR_vec_cmpv2dfv2di = 2146,
  CODE_FOR_vec_cmpv4sfv4si = 2147,
  CODE_FOR_vec_cmpv2div2di = 2148,
  CODE_FOR_vec_cmpv4siv4si = 2149,
  CODE_FOR_vec_cmpv8hiv8hi = 2150,
  CODE_FOR_vec_cmpv16qiv16qi = 2151,
  CODE_FOR_vec_cmpuv2div2di = 2152,
  CODE_FOR_vec_cmpuv4siv4si = 2153,
  CODE_FOR_vec_cmpuv8hiv8hi = 2154,
  CODE_FOR_vec_cmpuv16qiv16qi = 2155,
  CODE_FOR_absv2di2 = 2156,
  CODE_FOR_absv4si2 = 2157,
  CODE_FOR_absv8hi2 = 2158,
  CODE_FOR_absv16qi2 = 2159,
  CODE_FOR_negv2di2 = 2160,
  CODE_FOR_negv4si2 = 2161,
  CODE_FOR_negv8hi2 = 2162,
  CODE_FOR_negv16qi2 = 2163,
  CODE_FOR_msa_ldiv2di = 2164,
  CODE_FOR_msa_ldiv4si = 2165,
  CODE_FOR_msa_ldiv8hi = 2166,
  CODE_FOR_msa_ldiv16qi = 2167,
  CODE_FOR_movv2df = 2168,
  CODE_FOR_movv4sf = 2169,
  CODE_FOR_movv2di = 2170,
  CODE_FOR_movv4si = 2171,
  CODE_FOR_movv8hi = 2172,
  CODE_FOR_movv16qi = 2173,
  CODE_FOR_movmisalignv2df = 2174,
  CODE_FOR_movmisalignv4sf = 2175,
  CODE_FOR_movmisalignv2di = 2176,
  CODE_FOR_movmisalignv4si = 2177,
  CODE_FOR_movmisalignv8hi = 2178,
  CODE_FOR_movmisalignv16qi = 2179,
  CODE_FOR_msa_ld_d_f = 2180,
  CODE_FOR_msa_ld_w_f = 2181,
  CODE_FOR_msa_ld_d = 2182,
  CODE_FOR_msa_ld_w = 2183,
  CODE_FOR_msa_ld_h = 2184,
  CODE_FOR_msa_ld_b = 2185,
  CODE_FOR_msa_st_d_f = 2186,
  CODE_FOR_msa_st_w_f = 2187,
  CODE_FOR_msa_st_d = 2188,
  CODE_FOR_msa_st_w = 2189,
  CODE_FOR_msa_st_h = 2190,
  CODE_FOR_msa_st_b = 2191,
  CODE_FOR_reduc_smin_scal_v2df = 2192,
  CODE_FOR_reduc_smin_scal_v4sf = 2193,
  CODE_FOR_reduc_smin_scal_v2di = 2194,
  CODE_FOR_reduc_smin_scal_v4si = 2195,
  CODE_FOR_reduc_smin_scal_v8hi = 2196,
  CODE_FOR_reduc_smin_scal_v16qi = 2197,
  CODE_FOR_reduc_smax_scal_v2df = 2198,
  CODE_FOR_reduc_smax_scal_v4sf = 2199,
  CODE_FOR_reduc_smax_scal_v2di = 2200,
  CODE_FOR_reduc_smax_scal_v4si = 2201,
  CODE_FOR_reduc_smax_scal_v8hi = 2202,
  CODE_FOR_reduc_smax_scal_v16qi = 2203,
  CODE_FOR_reduc_umin_scal_v2di = 2204,
  CODE_FOR_reduc_umin_scal_v4si = 2205,
  CODE_FOR_reduc_umin_scal_v8hi = 2206,
  CODE_FOR_reduc_umin_scal_v16qi = 2207,
  CODE_FOR_reduc_umax_scal_v2di = 2208,
  CODE_FOR_reduc_umax_scal_v4si = 2209,
  CODE_FOR_reduc_umax_scal_v8hi = 2210,
  CODE_FOR_reduc_umax_scal_v16qi = 2211,
  CODE_FOR_reduc_plus_scal_v2df = 2212,
  CODE_FOR_reduc_plus_scal_v4sf = 2213,
  CODE_FOR_reduc_plus_scal_v2di = 2214,
  CODE_FOR_reduc_plus_scal_v4si = 2215,
  CODE_FOR_reduc_plus_scal_v8hi = 2216,
  CODE_FOR_reduc_plus_scal_v16qi = 2217,
  CODE_FOR_reduc_and_scal_v2di = 2218,
  CODE_FOR_reduc_ior_scal_v2di = 2219,
  CODE_FOR_reduc_xor_scal_v2di = 2220,
  CODE_FOR_reduc_and_scal_v4si = 2221,
  CODE_FOR_reduc_ior_scal_v4si = 2222,
  CODE_FOR_reduc_xor_scal_v4si = 2223,
  CODE_FOR_reduc_and_scal_v8hi = 2224,
  CODE_FOR_reduc_ior_scal_v8hi = 2225,
  CODE_FOR_reduc_xor_scal_v8hi = 2226,
  CODE_FOR_reduc_and_scal_v16qi = 2227,
  CODE_FOR_reduc_ior_scal_v16qi = 2228,
  CODE_FOR_reduc_xor_scal_v16qi = 2229
};

const unsigned int NUM_INSN_CODES = 2230;
#endif /* GCC_INSN_CODES_H */
