#ifndef IPTABLES_INTERNAL_H
#define IPTABLES_INTERNAL_H 1

/**
 * Program's own name and version.
 */
extern const char *program_name, *program_version;

extern int line;

#endif /* IPTABLES_INTERNAL_H */
